package com.tecnalia.across.ujse.api;

import com.tecnalia.across.ujse.model.UserInfo;
import java.io.IOException;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.tecnalia.across.ujse.model.UserWorkflowModel;
import com.tecnalia.across.ujse.model.store.UserWorkflowStore;
import com.tecnalia.across.ujse.repository.UserWorkflowRepository;
import com.tecnalia.across.ujse.service.WorkflowExecutionManagementService;
import com.tecnalia.across.ujse.utils.IdentityUtils;
import com.tecnalia.across.ujse.utils.JWTUtils;

import java.util.Optional;
import java.util.UUID;
import java.util.logging.Level;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;

@javax.annotation.Generated(value = "io.swagger.codegen.v3.generators.java.SpringCodegen", date = "2022-02-15T13:22:15.534Z[GMT]")
@RestController
public class WorkflowExecutionManagementApiController implements WorkflowExecutionManagementApi {

    private static final Logger log = LoggerFactory.getLogger(WorkflowExecutionManagementApiController.class);
    
    @Value("${citizen.fe.key}")
    String citizenFEKey = "";        


  /*  private final ObjectMapper objectMapper;

    private final HttpServletRequest request;

    @org.springframework.beans.factory.annotation.Autowired
    public WorkflowExecutionManagementApiController(ObjectMapper objectMapper, HttpServletRequest request) {
        this.objectMapper = objectMapper;
        this.request = request;
    }*/
    
    @Autowired
    private final WorkflowExecutionManagementService  workflowEMService;
    private  UserWorkflowRepository userWorkflowRepository;
    
    
    public WorkflowExecutionManagementApiController(WorkflowExecutionManagementService workflowEMService, UserWorkflowRepository userWorkflowRepository) {
        this.workflowEMService = workflowEMService;
        if (userWorkflowRepository == null)
            throw new IllegalArgumentException("userWorkflowRepository cannot be null.");       
        this.userWorkflowRepository = userWorkflowRepository;
       
    }
    
    
    public ResponseEntity<?> getServiceInfo(String serviceId) {
      
    	ResponseEntity<?> result = null;
        result=workflowEMService.getServiceInfo(serviceId);
        return result;
        
    }

    
    public ResponseEntity<List<UserWorkflowModel>> getUserWorkflowInstanceById(UUID workflowInstanceId) {
        
    	ResponseEntity<List<UserWorkflowModel>>  result = null;
		result=workflowEMService.getUserWorkflowInstanceById(workflowInstanceId);
		return result;
    }

 
    @Override
    public ResponseEntity<?> getWorkflowStatus(UUID workflowInstanceId) { 

     ResponseEntity<?> result = null;

     result=workflowEMService.getWorkflowStatus(workflowInstanceId);
     return result;
        
    }


    public ResponseEntity<?> killWorkflow(UUID workflowInstanceId) {
     
    	ResponseEntity<?> result = null;
        result=workflowEMService.killWorkflow(workflowInstanceId);
        return result;
    }


    public ResponseEntity<?> setServiceStatus(UUID workflowInstanceId, UUID phaseId, UUID actionId, String serviceId, String status) {
      
    	ResponseEntity<?> result = null;
        // Get userId
        UserInfo userInfo = IdentityUtils.getUserInfoFromAccessToken();
        String userId=userInfo.getUserId();
        result=workflowEMService.setServiceStatus(workflowInstanceId,phaseId,actionId,serviceId,status, null, userId, true);
        return result;
    }


    public ResponseEntity<?> setServiceStatusAsynchronous(String status, String token, String serviceOutput) {
      
    	ResponseEntity<?> result = null;
        // Check token is correct
        boolean correctToken = false;
        correctToken = this.verifyServiceToken(token, citizenFEKey);
        if (!correctToken) {
            String message = "Incorrect token";
            JSONObject messageJson = new JSONObject();
            messageJson.put("message", message);                       
            result=new ResponseEntity<> (messageJson.toString(),HttpStatus.UNAUTHORIZED);            
        } else {
            String payload = JWTUtils.getTokenPayload(token);
            JSONObject payloadJson = new JSONObject(payload);
            UUID workflowInstanceId = UUID.fromString((String)payloadJson.get("workflowInstanceId"));
            UUID phaseId = UUID.fromString((String)payloadJson.get("phaseId"));
            UUID actionId = UUID.fromString((String)payloadJson.get("actionId"));
            String serviceId = (String)payloadJson.get("serviceId");
            String userId = (String)payloadJson.get("userId");
            result=workflowEMService.setServiceStatus(workflowInstanceId,phaseId,actionId,serviceId,status,serviceOutput,userId,false);
        }
        return result;
    }


    public ResponseEntity<?> createUserWorkflowInstance(String countryOrigin, String countryDestination, String workflowType, String name) {
	
        ResponseEntity<?> result = null;
        try {
            result=workflowEMService.createUserWorkflowInstance(countryOrigin, countryDestination, workflowType,name);
        } catch (IOException ex) {
            java.util.logging.Logger.getLogger(WorkflowExecutionManagementApiController.class.getName()).log(Level.SEVERE, null, ex);
        }
		return result;
    }


    public ResponseEntity<?> executeService(UUID workflowInstanceId,  UUID phaseId, UUID actionId, String serviceId, String token, List<MultipartFile> inputData) throws IOException{
        ResponseEntity<?> result = null;

        if ((token != null) && (token.trim().length()>0)) {
            // Check token is correct
            boolean correctToken = false;
            correctToken = this.verifyServiceTokenAndContent(token, citizenFEKey, 
                workflowInstanceId, phaseId, actionId, serviceId);
            if (!correctToken) {
                String message = "Incorrect token";
                JSONObject messageJson = new JSONObject();
                messageJson.put("message", message);                       
                result=new ResponseEntity<> (messageJson.toString(),HttpStatus.UNAUTHORIZED);            
                return result;
            }
        }
        result=workflowEMService.executeService(workflowInstanceId, phaseId, actionId, serviceId, token, inputData);

        return result;
    }

  /*  public ResponseEntity<String> checkIfWorkflowHasFinished(UUID workflowInstanceId) {
        ResponseEntity<String> result = null;
        result=workflowEMService.checkIfWorkflowHasFinished(workflowInstanceId);
	return result;
    }
*/


    public ResponseEntity<String> deleteWorkflowInstance(UUID workflowInstanceId) {
        ResponseEntity<String> result = null;
        result=workflowEMService.deleteWorkflowInstanceId(workflowInstanceId);
	return result;
    }


    @Override
    public ResponseEntity<?> workflowName(UUID workflowInstanceId, String name) {
        ResponseEntity<?> result = null;
        result=workflowEMService.workflowName(workflowInstanceId, name);
	return result;
   
    }
    

    public ResponseEntity<?> setWorkflowStatus(UUID workflowInstanceId, String status) {
    
    	ResponseEntity<?> result = null;
        result=workflowEMService.setWorkflowStatus(workflowInstanceId, status);
        return result;
    }
    

    public ResponseEntity<?> setActionStatus(UUID workflowInstanceId, UUID phaseId, UUID actionId, String status) {
      
    	ResponseEntity<?> result = null;
        result=workflowEMService.setActionStatus(workflowInstanceId,phaseId,actionId,status);
        return result;
    }
    

    @Override
    public ResponseEntity<List<UserWorkflowModel>> getUserWorkflowsInstances(String countryOrigin, String countryDestination, String workflowType) {
        ResponseEntity<List<UserWorkflowModel>> result = null;
        result=workflowEMService.getUserWorkflowsInstances(countryOrigin,countryDestination,workflowType);
        return result;
    }
   

    ///////////////////////
    // Auxiliary functions
    ///////////////////////
    private boolean verifyServiceToken(String token, String secretKey){
        boolean correctTokenAndContent = false;
        if(JWTUtils.verifyToken(token, secretKey)) {
            // Check if the token payload is correct
            String payload = JWTUtils.getTokenPayload(token);
            JSONObject payloadJson = new JSONObject(payload);
            String payloadUserId = (String)payloadJson.get("userId");
            UUID workflowInstanceId = UUID.fromString((String)payloadJson.get("workflowInstanceId"));
            // Get userId for this workflow instance from DB and compare with the userId in the token
            Optional<UserWorkflowStore> bCheckExistsUserWorkflow=this.userWorkflowRepository.findByWorkflowInstanceId(workflowInstanceId);
            UserWorkflowStore userWorkflowStore;
            if (bCheckExistsUserWorkflow.isPresent()) {
                userWorkflowStore = (UserWorkflowStore)bCheckExistsUserWorkflow.get();
                if (payloadUserId.compareTo(userWorkflowStore.getUserId().toString()) == 0)
                    correctTokenAndContent = true;
            }
        }
        return correctTokenAndContent;
    }


    private boolean verifyServiceTokenAndContent(String token, String secretKey, 
            UUID workflowInstanceId, UUID phaseId, UUID actionId, String serviceId){
        boolean correctTokenAndContent = false;
        if(JWTUtils.verifyToken(token, secretKey)) {
            // Check if the token payload is correct
            String payload = JWTUtils.getTokenPayload(token);
            JSONObject payloadJson = new JSONObject(payload);
            String payloadUserId = (String)payloadJson.get("userId");
            if ((((String)payloadJson.get("workflowInstanceId")).compareTo(workflowInstanceId.toString()) == 0) &&
                (((String)payloadJson.get("phaseId")).compareTo(phaseId.toString()) == 0) &&
                (((String)payloadJson.get("actionId")).compareTo(actionId.toString()) == 0) &&
                (((String)payloadJson.get("serviceId")).compareTo(serviceId) == 0)) {
                // Get userId for this workflow instance from DB and compare with the userId in the token
                Optional<UserWorkflowStore> bCheckExistsUserWorkflow=this.userWorkflowRepository.findByWorkflowInstanceId(workflowInstanceId);
                UserWorkflowStore userWorkflowStore;
                if (bCheckExistsUserWorkflow.isPresent()) {
                    userWorkflowStore = (UserWorkflowStore)bCheckExistsUserWorkflow.get();
                    if (payloadUserId.compareTo(userWorkflowStore.getUserId().toString()) == 0)
                        correctTokenAndContent = true;
                }
            }
        }
        return correctTokenAndContent;
    }

   
}