/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.tecnalia.across.ujse.external;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.tecnalia.across.ujse.exception.RestContentNotSupportedException;
import com.tecnalia.across.ujse.exception.RestMethodNotSupportedException;
import com.tecnalia.across.ujse.model.OutputDataElem;
import com.tecnalia.across.ujse.utils.HttpPostMultipart;
import com.tecnalia.across.ujse.utils.HttpUtils;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.commons.codec.binary.Base64;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.web.multipart.MultipartFile;

/**
 *
 * @author tecnalia
 */
public class RestServiceInvocation {

    public List<OutputDataElem> invokeService(String serviceUrl, String channel, 
            String token, List<MultipartFile> inputData, 
            String method, String contentType, HashMap<String, String> serviceInputTypes,
            HashMap<String, Boolean> serviceInputMultiple) 
            throws IOException, UnsupportedEncodingException, 
            RestMethodNotSupportedException, RestContentNotSupportedException{
        List<OutputDataElem> serviceOutput = new ArrayList<OutputDataElem>();

        //TODO: Remove this traces
        for (MultipartFile mf : inputData) {
            //System.out.println("name="+mf.getName());
            //System.out.println("originalFilename="+mf.getOriginalFilename());
            /*
            try {
                String inputStr = new String(mf.getBytes(), StandardCharsets.UTF_8);
                String inputStrDecoded = new String(Base64.decodeBase64(mf.getBytes()));
                System.out.println("inputStr="+inputStr);
                System.out.println("inputStrDecoded="+inputStrDecoded);
            } catch(IOException e) {
                e.printStackTrace();
            }*/
            //System.out.println("contentType="+mf.getContentType());
            //System.out.println("size="+mf.getSize());
        }
        
        if (method.compareToIgnoreCase("GET") == 0) {
            //Invoke the service
            serviceOutput =  sendGETRequest(serviceUrl, channel, token, inputData);
        }
        else if ((method.compareToIgnoreCase("POST") == 0) || (method.compareToIgnoreCase("PUT") == 0)){
            if (contentType.compareToIgnoreCase("application/json") == 0) {
                //Invoke the service
                serviceOutput =  sendPostPutJsonRequest(serviceUrl, channel, token, inputData, method, serviceInputMultiple);                
            }
            else if (contentType.compareToIgnoreCase("multipart/form-data") == 0) {
                //Invoke the service
                serviceOutput =  sendPostPutMultipartRequest(serviceUrl, channel, token, inputData, method, serviceInputTypes);                                
            }
            else {
                throw new RestContentNotSupportedException("The " + contentType + "content type is not supported");
            }            
        } else {
            throw new RestMethodNotSupportedException("The " + method + " method is not supported");
        }
        
        return serviceOutput;
    }
    
    
    public List<OutputDataElem> sendGETRequest(String serviceUrl, String channel, 
            String token, List<MultipartFile> inputData) throws IOException{
        //TODO: Quitar esta URL
        //serviceUrl = "http://across:8085/DataUsage/Pip/1.0/admin/api/pruebaAcross";
        
        //Compose URL with input data
        int i = 0;
        String unionStr = "?";
        for (MultipartFile mf : inputData) {
            String inputStr = new String(mf.getBytes(), StandardCharsets.UTF_8);
            inputStr = inputStr.trim();
            serviceUrl = serviceUrl + unionStr + mf.getOriginalFilename() + "=" + URLEncoder.encode(inputStr, StandardCharsets.UTF_8.toString());                    
            i++;
            if (i > 0)
                unionStr = "&";
        }
        // TODO: How do we pass the token?
        serviceUrl = serviceUrl + unionStr + "across_async_token=" + token;
                
        //Invoke the service
        HttpUtils httpUtil=new HttpUtils();
        String serviceOutputStr =  httpUtil.sendGETRequest(serviceUrl, null);
        
        //TODO: quitar esta traza
        System.out.println("Invoke service provider with GET "+serviceUrl);
        //System.out.println("GET Output "+serviceOutputStr);

        //Compose output data
        final ObjectMapper objectMapper = new ObjectMapper();
        OutputDataElem[] searchOutputOjbArray = objectMapper.readValue(serviceOutputStr, OutputDataElem[].class);

        return Arrays.asList(searchOutputOjbArray);
    }
    
    
    public List<OutputDataElem> sendPostPutJsonRequest(String serviceUrl, String channel, 
            String token, List<MultipartFile> inputData, String method, HashMap<String, Boolean>  serviceInputMultiple) throws IOException{
        //TODO: Quitar esta URL
        //serviceUrl = "http://across:8085/DataUsage/Pip/1.0/admin/api/pruebaAcrossJson";
        //serviceUrl = "http://across:8085/DataUsage/Pip/1.0/admin/api/pruebaAcross";
        // TODO: How do we pass the token?
        serviceUrl = serviceUrl + "?across_async_token=" + token;
        //Compose JSON with input data
        JSONObject inputJson = new JSONObject();
        for (MultipartFile mf : inputData) {
            String inputStr = new String(mf.getBytes(), StandardCharsets.UTF_8);
            inputStr = inputStr.trim();
            if (serviceInputMultiple.getOrDefault(mf.getOriginalFilename(), false)) {
                //Multiple values => Array type
                if(inputJson.has(mf.getOriginalFilename())) {
                    JSONArray multValsArray = inputJson.getJSONArray(mf.getOriginalFilename());
                    multValsArray.put(inputStr);
                } else {
                    //Create new array
                    JSONArray multValsArray = new JSONArray();
                    multValsArray.put(inputStr);
                    inputJson.put(mf.getOriginalFilename(), multValsArray);
                }
            } else {
                inputJson.put(mf.getOriginalFilename(), inputStr);
            }
        }
            
        //Invoke the service
        String serviceOutputStr;
        HttpUtils httpUtil=new HttpUtils();
        if (method.compareToIgnoreCase("POST")==0)
            serviceOutputStr =  httpUtil.sendPOSTRequest(serviceUrl, null, inputJson.toString());
        else 
            serviceOutputStr =  httpUtil.sendPUTRequest(serviceUrl, null, inputJson.toString());

        //Compose output data
        final ObjectMapper objectMapper = new ObjectMapper();
        OutputDataElem[] searchOutputOjbArray = objectMapper.readValue(serviceOutputStr, OutputDataElem[].class);

        return Arrays.asList(searchOutputOjbArray);
    }
    
    
    public List<OutputDataElem> sendPostPutMultipartRequest(String serviceUrl, String channel, 
            String token, List<MultipartFile> inputData, 
            String method, HashMap<String, String> serviceInputTypes) throws IOException{            
        //TODO: Quitar esta URL
        //serviceUrl = "http://across:8080/DataUsage/Pip/1.0/admin/api/pruebaMultipartAcross";
        // TODO: How do we pass the token?
        serviceUrl = serviceUrl + "?across_async_token=" + token;
        //Invoke the service
        String serviceOutputStr;
        // Set header
        Map<String, String> headers = new HashMap<>();
        HttpPostMultipart multipart = new HttpPostMultipart(serviceUrl, "utf-8", headers, method);
        //Compose multipart data
        JSONObject inputJson = new JSONObject();
        for (MultipartFile mf : inputData) {
            String inputType = serviceInputTypes.get(mf.getOriginalFilename());
            if (inputType != null) {
                if (inputType.compareToIgnoreCase("string")==0) {
                    String inputStr = new String(mf.getBytes(), StandardCharsets.UTF_8);
                    inputStr = inputStr.trim();
                    // Add form field
                    multipart.addFormField(mf.getOriginalFilename(), inputStr);
                } else if (inputType.compareToIgnoreCase("file")==0) {
                    // Add file
                    multipart.addFilePartContent(mf.getOriginalFilename(), mf.getBytes(), mf.getContentType());                    
                }
            }
        }
        
        // Get Result
        serviceOutputStr = multipart.finish();
                
        //Compose output data
        final ObjectMapper objectMapper = new ObjectMapper();
        OutputDataElem[] searchOutputOjbArray = objectMapper.readValue(serviceOutputStr, OutputDataElem[].class);

        //TODO: Quitar esto, son pruebas
        /*Date date = new Date();
        long time_milli = date.getTime();
        for(OutputDataElem outEl: searchOutputOjbArray) {
            if(outEl.getType()==OutputDataElem.TypeEnum.FILE) {
                try {
                    String filePath = "/home/tecnalia/encoded_"+outEl.getOutputId()+"_"+time_milli;
                    FileOutputStream fileOutputStream = new FileOutputStream(filePath);
                    fileOutputStream.write(outEl.getContent());
                    fileOutputStream.close();
                    filePath = "/home/tecnalia/"+outEl.getOutputId()+"_"+time_milli;
                    byte[] decodedContent = Base64.decodeBase64(outEl.getContent());
                    fileOutputStream = new FileOutputStream(filePath);
                    fileOutputStream.write(decodedContent);
                    fileOutputStream.close();
                    if(outEl.getContentType().compareToIgnoreCase("application/pdf") == 0)
                        System.out.println("encoded Content="+new String(outEl.getContent(), 0, 10));
                } catch (IOException e) {
                    System.out.println(e.getMessage());
                }                
            }
        }*/
        
        return Arrays.asList(searchOutputOjbArray);
    }
}
