/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.tecnalia.across.ujse.external;

import com.tecnalia.across.ujse.exception.DataConsentRequiredException;
import com.tecnalia.across.ujse.exception.MissingInputsException;
import com.tecnalia.across.ujse.exception.PolicyNotApplyException;
import com.tecnalia.across.ujse.exception.RestContentNotSupportedException;
import com.tecnalia.across.ujse.exception.RestMethodNotSupportedException;
import com.tecnalia.across.ujse.model.ExecuteStepOutput;
import com.tecnalia.across.ujse.model.InputData;
import com.tecnalia.across.ujse.model.InputDataDescription;
import com.tecnalia.across.ujse.model.MultilingValue;
import com.tecnalia.across.ujse.model.InputSource;
import com.tecnalia.across.ujse.model.LocaleDescription;
import com.tecnalia.across.ujse.model.OutputDataElem;
import com.tecnalia.across.ujse.model.ServiceInfo;
import com.tecnalia.across.ujse.model.ServiceStore;
import com.tecnalia.across.ujse.utils.HttpUtils;
import com.tecnalia.across.ujse.utils.WorkflowUtils;


import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.web.multipart.MultipartFile;

/**
 *
 * @author tecnalia
 */
public class ServiceCatalogueENG {
    
    
    public JSONObject getServiceInfoFromCatalog(String serviceId, String getServiceInfoUrl) throws IOException{
        HttpUtils httpUtil=new HttpUtils();
        
        //Get service info
        String serviceObjStr=httpUtil.sendGETRequest(getServiceInfoUrl +"?identifier="+java.net.URLEncoder.encode(serviceId, "UTF-8"),null);
        JSONObject serviceObjJson = new JSONObject(serviceObjStr);
        
        return serviceObjJson;
    }  
    
    public ServiceStore getServiceInfo(String serviceId, String getServiceInfoUrl) throws IOException {
        ServiceStore serviceStore = new ServiceStore();        
   
        JSONObject serviceObjJson = getServiceInfoFromCatalog(serviceId, getServiceInfoUrl);
        
        //Get service attributes     
        //get channel  hasInfo-> hasChannel-> type
        String channel = "";
        String contact = "";
        String location = "";
        JSONObject objhasInfo=(JSONObject)serviceObjJson.get("hasInfo");
        location = objhasInfo.getString("spatial");
        serviceStore.setLocation(location);
        JSONArray channelArray=(JSONArray)objhasInfo.get("hasChannel");
        channel = getChannel(channelArray);
        contact = getContactInfo((JSONObject)objhasInfo.get("hasContactPoint"), channel);
        serviceStore.setChannel(channel);
        serviceStore.setContact(contact);
       
        //get hasServiceInstance
        JSONObject objHasServiceInstance=(JSONObject)serviceObjJson.get("hasServiceInstance");
        
        //Inputs: hasServiceInstance-> dataset[]-> dataMapping[]-> property, name, inputType, required
        serviceStore.inputDataRequired(new ArrayList<InputData>());
        JSONArray datasetArrJson=(JSONArray)objHasServiceInstance.get("dataset");
        for (Object dataset : datasetArrJson) {
            JSONObject datasetJson = (JSONObject)dataset;
            JSONArray datamappingArrJson=(JSONArray)datasetJson.get("dataMapping");
            for (Object dataMapping : datamappingArrJson) {
                JSONObject dataMappingJson = (JSONObject)dataMapping;
                InputData input1=new InputData();
                input1.setInputId(dataMappingJson.getString("property"));
                input1.setType(dataMappingJson.getString("inputType"));
                if(dataMappingJson.has("multiple"))
                    input1.setMultiple(dataMappingJson.getBoolean("multiple"));
                input1.setRequired(dataMappingJson.getBoolean("required"));
                //Get source info
                InputSource inputSource = new InputSource();
                if (dataMappingJson.has("source")) {
                    String distributionId = dataMappingJson.getString("source");
                    JSONArray distributionArrJson=(JSONArray)datasetJson.get("distribution");
                    for (Object dist : distributionArrJson) {
                        JSONObject distJson = (JSONObject)dist;
                        if(distJson.getString("distributionId").compareTo(distributionId) == 0) {
                            inputSource.setAccessUrl(distJson.getString("accessUrl"));
                            inputSource.setFormat(distJson.getString("format"));
                            //Get list of locale and description
                            JSONArray descJsonArray=distJson.getJSONArray("description");                   
                            ArrayList<LocaleDescription> descriptionList = new ArrayList<LocaleDescription>();
                            for (Object descObj : descJsonArray) {
                                JSONObject descJson = (JSONObject)descObj;
                                LocaleDescription localeDesc = new LocaleDescription();
                                localeDesc.setDescription(descJson.getString("description"));
                                localeDesc.setLocale(descJson.getString("locale"));
                                descriptionList.add(localeDesc);
                            }
                            inputSource.setDescription(descriptionList);
                        }
                    }
                }
                input1.setSource(inputSource);
                // Get readonly and datamap
                boolean readonly = false;
                String datamap = "";
                if (dataMappingJson.has("readonly"))
                    readonly = dataMappingJson.getBoolean("readonly");
                if (dataMappingJson.has("datamap"))
                    datamap = dataMappingJson.getString("datamap");
                input1.setReadonly(readonly);
                input1.setDatamap(datamap);
                
                //Get descriptions array
                // Get inputs name, description and language from dataMapping -> description
                String inputTitle = dataMappingJson.getString("name");
                JSONArray descJsonArray=dataMappingJson.getJSONArray("description");                   
                //Get list of locale, title and description
                ArrayList<InputDataDescription> descriptionList = new ArrayList<InputDataDescription>();
                for (Object descObj : descJsonArray) {
                    JSONObject descJson = (JSONObject)descObj;
                    InputDataDescription inputDataDesc = new InputDataDescription();
                    inputDataDesc.setTitle(inputTitle);
                    if (descJson.has("title") && descJson.getString("title").trim().length()> 0)
                        inputDataDesc.setTitle(descJson.getString("title"));
                    inputDataDesc.setDescription(descJson.getString("description"));
                    inputDataDesc.setLocale(descJson.getString("locale"));
                    descriptionList.add(inputDataDesc);
                }
                input1.setDescription(descriptionList);
                
                serviceStore.addInputDataRequiredItem(input1);            
            }
        }
        
        
        // Get inputs name, description and language from hasInfo -> hasInput
        // TODO: Remove this
        /*
        JSONArray inputDataReqJson=(JSONArray)objhasInfo.get("hasInput");        
        for (Object inputElem : inputDataReqJson) {
            JSONObject inputElemJson = (JSONObject)inputElem;
            String inputId = inputElemJson.getString("identifier");
            String inputTitle = inputElemJson.getString("title");
            JSONArray descJsonArray=inputElemJson.getJSONArray("description");                   
            //Get list of locale, title and description
            ArrayList<InputDataDescription> descriptionList = new ArrayList<InputDataDescription>();
            for (Object descObj : descJsonArray) {
                JSONObject descJson = (JSONObject)descObj;
                InputDataDescription inputDataDesc = new InputDataDescription();
                inputDataDesc.setTitle(inputTitle);
                if (descJson.has("title") && descJson.getString("title").trim().length()> 0)
                    inputDataDesc.setTitle(descJson.getString("title"));
                inputDataDesc.setDescription(descJson.getString("description"));
                inputDataDesc.setLocale(descJson.getString("locale"));
                descriptionList.add(inputDataDesc);
            }    
            
            for (InputData inputDataReq :serviceStore.getInputDataRequired()) {
                if(inputDataReq.getInputId().compareTo(inputId) == 0) {
                    inputDataReq.setDescription(descriptionList);
                }
            }
        }*/
       
        
        return serviceStore;
    }   
    

    public HashMap<String, ArrayList<MultilingValue>> getServiceDescriptions(String serviceId, String getServiceInfoUrl) throws IOException {
        HashMap<String, ArrayList<MultilingValue>> nameDesHM = new HashMap<>();
        ArrayList<MultilingValue> nameList = new ArrayList<>();
        ArrayList<MultilingValue> descriptionList = new ArrayList<>();
        nameDesHM.put("name", nameList);
        nameDesHM.put("description", descriptionList);
   
        JSONObject serviceObjJson = getServiceInfoFromCatalog(serviceId, getServiceInfoUrl);
        
        //Get service descriptions
        String inputTitle = serviceObjJson.getString("title");
        JSONObject objhasInfo=(JSONObject)serviceObjJson.get("hasInfo");        
        JSONArray descJsonArray=(JSONArray)objhasInfo.get("description");        
        //Get list of locale, title and description
        for (Object descObj : descJsonArray) {
            JSONObject descJson = (JSONObject)descObj;
            MultilingValue multiName = new MultilingValue();
            MultilingValue multiDesc = new MultilingValue();
            multiName.setValue(inputTitle);
            if (descJson.has("title") && descJson.getString("title").trim().length()> 0)
                multiName.setValue(descJson.getString("title"));
            multiDesc.setValue(descJson.getString("description"));
            multiName.setLocale(descJson.getString("locale"));
            multiDesc.setLocale(descJson.getString("locale"));
            nameList.add(multiName);
            descriptionList.add(multiDesc);
        }    
        
        return nameDesHM;
    }   


    public List<ServiceInfo>  getPersonalDataServices(String getServicesUrl,ArrayList<String> servicesIdList,UUID workflowInstanceId) throws IOException{
        List<ServiceInfo> list = new ArrayList<>();
        HttpUtils httpUtil=new HttpUtils();
        
        if (servicesIdList.size()>0){
            String url=getServicesUrl;
            for (String serviceId: servicesIdList){
                //url+="identifier="+serviceId+"&";  
                String [] aServices=serviceId.split(";");
                for (int i=0;i<aServices.length;i++){
                    String service=aServices[i];
                    url+="identifier="+java.net.URLEncoder.encode(service, "UTF-8")+"&";    
                }
            }
            url=url.substring(0, url.lastIndexOf("&"));
            String output=httpUtil.sendGETRequest(url, null);

            if ((output!=null)|| (output.length()>0)){
                JSONArray serviceInfoJson=new JSONArray(output);
                for (Object serviceJson : serviceInfoJson) {
                    JSONObject obj = (JSONObject) serviceJson;
                    JSONArray jPersonalData= obj.getJSONArray("isPersonalDataHandling");
                    if (jPersonalData.length()>0){
                        if ((JSONObject) jPersonalData.get(0)!=null){
                            JSONObject jObjPersonalData=(JSONObject) jPersonalData.get(0);
                            JSONArray jRequiredDatasetsArr=jObjPersonalData.getJSONArray("requiredDatasets");
                            //if (jObjPersonalData.getString("purposeId")!=null)
                            if (jRequiredDatasetsArr.length()>0)
                            {
                                ServiceInfo serviceTransp=new ServiceInfo();
                                serviceTransp.setServiceId((String)obj.get("identifier"));
                                serviceTransp.setServiceName((String)obj.get("title"));
                                serviceTransp.setWorkflowInstanceId(workflowInstanceId);
                                list.add(serviceTransp);
                            }
                        }
                    }
                }
            }
        }

       return list;         
    }    
    
    
    public ExecuteStepOutput executeService(String serviceId, String token, List<MultipartFile> inputData, 
            String getServiceInfoUrl, 
            String checkConsentServiceUrl, String consentInfoServiceUrl,
            String logDataUsageServiceUrl,String getPolicyPatternsUrl,
            String datausagecontrolEnforcementUri,
            String oauthUrl, String oauthTokenUrlPart)
            throws IOException, MissingInputsException, DataConsentRequiredException,
            RestMethodNotSupportedException, RestContentNotSupportedException{
        ExecuteStepOutput executeOutput=new ExecuteStepOutput();
        List<OutputDataElem> serviceOutput = new ArrayList<OutputDataElem>();
        //Invoke service
        // - if Consent missing -> consent missing
        // - if input data required -> info X required
        // - if invocation OK -> status Finished

        // Invoke Service Catalogue and get following info:
        //      - if Data Consent is required
        //      - Inputs
        JSONObject serviceObjJson = getServiceInfoFromCatalog(serviceId, getServiceInfoUrl);
        //String serviceName=serviceObjJson.getString("title");
        JSONObject objhasInfo=(JSONObject)serviceObjJson.get("hasInfo");
        JSONArray channelArray=(JSONArray)objhasInfo.get("hasChannel");
        String channel = getChannel(channelArray);
        
        
        //Check if Data Consent is required
        boolean dataConsentRequired = checkDataConsentRequired(serviceObjJson);

        // Check with TD if consent given
        TransparencyDashboard transparencyDashboard = new TransparencyDashboard();              
        if(dataConsentRequired) {
            //Tranparency Dashboard invocation, check if service has consent by user
            boolean bConsentStatus = transparencyDashboard.checkServiceConsentStatusByUser(checkConsentServiceUrl, oauthUrl, oauthTokenUrlPart);
            if (!bConsentStatus)
                throw new DataConsentRequiredException("Data Consent for service " +
                    serviceObjJson.getString("title") + " is required");

        }        
                
                        
        //Check if inputs required have been provided
        HashMap<String, String> serviceInputTypes = new HashMap<String, String>();
        HashMap<String, Boolean>  serviceInputMultiple  = new HashMap<String, Boolean>();
        String missingInputsStr = "The following inputs are missing: ";
        boolean missingInputs = false;        
        //Inputs: hasServiceInstance-> dataset[]-> dataMapping[]-> property, name, inputType, required
        JSONObject objHasServiceInstance=(JSONObject)serviceObjJson.get("hasServiceInstance");
        JSONArray datasetArrJson=(JSONArray)objHasServiceInstance.get("dataset");
        for (Object dataset : datasetArrJson) {
            JSONObject datasetJson = (JSONObject)dataset;
            JSONArray datamappingArrJson=(JSONArray)datasetJson.get("dataMapping");
            for (Object dataMapping : datamappingArrJson) {
                JSONObject dataMappingJson = (JSONObject)dataMapping;
                String inputId = dataMappingJson.getString("property");
                String inputName = dataMappingJson.getString("name");
                String inputType = dataMappingJson.getString("inputType");
                if (inputType.compareToIgnoreCase("file")==0)
                    serviceInputTypes.put( inputId,"file");
                else 
                    serviceInputTypes.put( inputId,"string");
                if(dataMappingJson.has("multiple"))
                    serviceInputMultiple.put(inputId, dataMappingJson.getBoolean("multiple"));
                else
                    serviceInputMultiple.put(inputId, false);  
                boolean inputRequired = dataMappingJson.getBoolean("required");
                if(!inputRequired)
                    continue;
                boolean inputProvided = false;
                for (MultipartFile mf : inputData) {
                    if(mf.getOriginalFilename().compareTo(inputId) == 0){
                        inputProvided = true;
                        break;
                    }
                }
                if(inputRequired && !inputProvided) {
                    missingInputs = true;
                    missingInputsStr += inputId + " ";
                }
                
            }
        }
        if(missingInputs){
            throw new MissingInputsException(missingInputsStr);
        }
        
        
        //Remove inputs which do not have consent given
        ArrayList<MultipartFile> filteredInputData = removeInputsWithNoConsent(inputData, 
                consentInfoServiceUrl, serviceId, oauthUrl, oauthTokenUrlPart);
        
        
        //Check data usage policies
        if(dataConsentRequired) {
            DataUsageControl dataUsage=new DataUsageControl();
            JSONArray jResult = dataUsage.checkDataUsagePolicies(getPolicyPatternsUrl,datausagecontrolEnforcementUri);  
            if ((jResult!=null) && (jResult.length()!=0))
                    throw new PolicyNotApplyException("The following policies are not applied, go to the Transparency Dashboard to solve them " +
                        jResult.toString());
        }        
            
        //get hasServiceInstance-> endpointConnector-> endpoint-> accessURL
        JSONObject objConn=(JSONObject)objHasServiceInstance.get("endpointConnector");
        JSONObject objConnEndpoint=(JSONObject)objConn.get("endpoint");
        String serviceUrlRoot=(String)objConnEndpoint.getString("accessURL");
        String servicePathMethod=(String)objConnEndpoint.get("path");
        String[] servicePathMethodArr = servicePathMethod.split("\\s+");
        if((serviceUrlRoot.charAt(serviceUrlRoot.length()-1)== '/') && 
            (servicePathMethodArr[0].charAt(0)) == '/') 
            serviceUrlRoot = serviceUrlRoot.substring(0, serviceUrlRoot.length()-1);
        String serviceUrl = serviceUrlRoot + servicePathMethodArr[0];
        // Parse method and content-type: [POST, Content-Type:application/json]
        String defaultMethod = "POST";
        String defaultContentType = "application/json";
        String method = defaultMethod;
        String contentType = defaultContentType;
        int methodPosStart = servicePathMethod.indexOf("[");
        if (methodPosStart>=0) {
            int methodPosEnd = servicePathMethod.indexOf("]");
            if (methodPosEnd >=0) {
                String methodContentStr = servicePathMethod.substring(methodPosStart+1, methodPosEnd);
                String[] methodContentArr = methodContentStr.split(",");
                if (methodContentArr.length == 1) {
                    if (methodContentArr[0].toUpperCase().startsWith("CONTENT")) {
                        String[] contentArr = methodContentArr[0].split(":");
                        if (contentArr.length == 2)
                            contentType = ((contentArr[1].trim().length() > 0) ? contentArr[1].trim() : defaultContentType);
                    } else {
                        method =  ((methodContentArr[0].trim().length() > 0) ? methodContentArr[0].trim() : defaultMethod);
                    }
                }
                if (methodContentArr.length == 2) {
                    method =  ((methodContentArr[0].trim().length() > 0) ? methodContentArr[0].trim() : defaultMethod);
                    String[] contentArr = methodContentArr[1].split(":");
                    if (contentArr.length == 2)
                        contentType = ((contentArr[1].trim().length() > 0) ? contentArr[1].trim() : defaultContentType);
                }
            }
        }
        
        if ((method.compareToIgnoreCase("GET") != 0) &&  
                (method.compareToIgnoreCase("POST") != 0) &&
                (method.compareToIgnoreCase("PUT") != 0))
            throw new RestMethodNotSupportedException("The " + method + " method is not supported");
        
        if ((contentType.compareToIgnoreCase("application/json") != 0) &&  
                (contentType.compareToIgnoreCase("multipart/form-data") != 0))
            throw new RestContentNotSupportedException("The " + contentType + "content type is not supported");

        //Invoke Service
        RestServiceInvocation restServiceInvocation = new RestServiceInvocation();
        serviceOutput = restServiceInvocation.invokeService(serviceUrl, channel, token, filteredInputData, method, contentType, 
                serviceInputTypes, serviceInputMultiple);
        
        //Log data usage event in Transparency Dashboard
        if(dataConsentRequired)
            transparencyDashboard.logPersonalDataUsage(logDataUsageServiceUrl, oauthUrl, oauthTokenUrlPart);

        //Set step status
        if (channel.compareTo("RESTService") == 0) {
            executeOutput.setStepStatus("finished");
        } else {
            executeOutput.setStepStatus(WorkflowUtils.SERV_INITIAL_STATUS);
        }
        executeOutput.setOutputData(serviceOutput);
        return executeOutput;        
    }
    

    /////////////////////
    // Helping methods
    /////////////////////    
    private String getChannel(JSONArray channelArray) {
        String channel = "";
        for (Object channelElem : channelArray) {
            JSONObject channelObj = (JSONObject)channelElem;
            channel=channelObj.getString("type");
            if((channel.compareTo("RESTService") == 0) ||
                 (channel.compareTo("AsyncRESTService") == 0)   )
                 break;
        }
        
        return channel;
    }
    

    private String getContactInfo(JSONObject hasContactPoint, String channel){
        String contact = "";
        
        if((channel.compareTo("RESTService") == 0) ||
            (channel.compareTo("AsyncRESTService") == 0)   ){
             contact = "";
        } else {
            switch(channel.toUpperCase()) {
                case "E-MAIL":
                    contact = hasContactPoint.getString("email");
                    break;
                case "TELEPHONE":
                    contact = hasContactPoint.getString("telephone");
                    break;
                case "HOMEPAGE":
                    contact = hasContactPoint.getString("url");
                    break;
                default:
                    contact = "";
                    break;
            }
        }
        return contact;        
    }

    private boolean checkDataConsentRequired(JSONObject serviceObjJson) {
        boolean dataConsentRequired = false;        
        JSONArray jPersonalData= serviceObjJson.getJSONArray("isPersonalDataHandling");
        if (jPersonalData.length()>0){
            if ((JSONObject) jPersonalData.get(0)!=null){

                JSONObject jObjPersonalData=(JSONObject) jPersonalData.get(0);
                JSONArray jRequiredDatasetsArr=jObjPersonalData.getJSONArray("requiredDatasets");

                //if (jObjPersonalData.getString("purposeId")!=null)
                if (jRequiredDatasetsArr.length()>0)
                {
                    dataConsentRequired = true;
                }
            }
        }
        return dataConsentRequired;
    }
    
    
    private ArrayList<MultipartFile> removeInputsWithNoConsent (List<MultipartFile> inputData, 
            String consentInfoServiceUrl, 
            String serviceId, String oauthUrl, String oauthTokenUrlPart) throws IOException{
        ArrayList<MultipartFile> filteredInputData = new ArrayList<MultipartFile>();
        List<String> alreadyCheckedInputs = new ArrayList<String>();
        TransparencyDashboard transparencyDashboard = new TransparencyDashboard();
        JSONArray persDataArray = transparencyDashboard.getConsentInfoForService(consentInfoServiceUrl, serviceId, oauthUrl, oauthTokenUrlPart);
        for (MultipartFile mf : inputData) {
            boolean inputFoundInConsents = false;
            if(persDataArray.length() >0) {
                for (Object persDataElem : persDataArray) {
                    JSONObject persDataElemJson = (JSONObject)persDataElem;
                    String personalDataId = persDataElemJson.getString("personalDataId");
                    if(mf.getOriginalFilename().compareTo(personalDataId) == 0){
                        inputFoundInConsents = true;
                        Boolean grantConsent = persDataElemJson.getBoolean("grantConsent");
                        if(grantConsent)  {
//                        if(grantConsent && (!alreadyCheckedInputs.contains(personalDataId)))  {
                            //If consent given, add input
                            filteredInputData.add(mf);
                            alreadyCheckedInputs.add(personalDataId);
                        }
                    }
                }
            }
            if(!inputFoundInConsents) {
                filteredInputData.add(mf);
            }
        }
        return filteredInputData;
    
    }
    
}
