package com.tecnalia.across.ujse.service;


import com.tecnalia.across.ujse.model.LogResponse;
import com.tecnalia.across.ujse.model.StatisticsInfo;
import com.tecnalia.across.ujse.model.UserInfo;
import java.io.IOException;
import java.util.List;

import javax.transaction.Transactional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import com.tecnalia.across.ujse.model.store.LogStore;
import com.tecnalia.across.ujse.repository.LogRepository;
import com.tecnalia.across.ujse.utils.IdentityUtils;
import java.time.Instant;
import java.util.UUID;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;




@Service
public class LogManagementService {
	
    private static final Logger log = LoggerFactory.getLogger(LogManagementService.class);
	
    @Autowired
    private  LogRepository logRepository;
	
    @Autowired
    public LogManagementService(LogRepository logRepository) throws IllegalArgumentException {
        if (logRepository == null)
            throw new IllegalArgumentException("logRepository cannot be null.");       

        this.logRepository = logRepository;
    }
	 
	
    @Transactional
    public ResponseEntity<String>  saveLog (String description, 
        String workflowInstanceId, String phaseId, String actionId, String serviceId) throws IOException {
        
        // Get userId
        UserInfo userInfo = IdentityUtils.getUserInfoFromAccessToken();
        String userId=userInfo.getUserId();
        ResponseEntity<String> result=null; 
        try {
            saveLogInDB(description, workflowInstanceId, phaseId, actionId, serviceId, userId, true);
            String message="log has been registered";
            result=new ResponseEntity<String>(message,HttpStatus.OK);
        }
        catch (Exception e) {
            String message="An error happened during workflow upload process";
            result=new ResponseEntity<String>(message,HttpStatus.BAD_REQUEST);
        }
        return result;
    }


    public ResponseEntity<LogResponse>  getAllLogsPaged(String workflowInstanceId, String phaseId, String actionId, String serviceId,
            int pageNo, int pageSize){

        // Get userId
        UserInfo userInfo = IdentityUtils.getUserInfoFromAccessToken();
        String userId=userInfo.getUserId();

        ResponseEntity<LogResponse> result = null;
        List<LogStore> logsList=null;
        LogResponse logResponse = new LogResponse();

        try {
            // create Pageable instance
            Pageable pageable = PageRequest.of(pageNo, pageSize, Sort.by("createdOn").descending());
            Page<LogStore> logsPage = null;
            if(workflowInstanceId!=null && phaseId!=null && actionId!=null) {
                if (serviceId!=null) {
                    logsPage = (Page<LogStore>) this.logRepository.findByUserIdAndWorkflowInstanceIdAndPhaseIdAndActionIdAndServiceIdPage(userId, 
                        workflowInstanceId, phaseId, actionId, serviceId, pageable);                
                    
                } else {
                    logsPage = (Page<LogStore>) this.logRepository.findByUserIdAndWorkflowInstanceIdAndPhaseIdAndActionIdPage(userId, 
                        workflowInstanceId, phaseId, actionId, pageable);                                    
                }
            } else {
                logsPage = (Page<LogStore>) this.logRepository.findAllByUserIdAndWorkflowInstanceIdAndPhaseIdAndActionIdAndServiceIdPage(userId, 
                    workflowInstanceId, phaseId, actionId, serviceId, pageable);                
            }
            
            // get content for page object
            logsList = logsPage.getContent();
            logResponse.setContent(logsList);
            logResponse.setPageNo(logsPage.getNumber());
            logResponse.setPageSize(logsPage.getSize());
            logResponse.setTotalElements(logsPage.getTotalElements());
            logResponse.setTotalPages(logsPage.getTotalPages());
            logResponse.setLast(logsPage.isLast());

            result=new ResponseEntity<LogResponse>(logResponse,HttpStatus.OK);
        }
        catch(Exception e) {
            result=new ResponseEntity<>(logResponse,HttpStatus.NOT_FOUND);
        }
        return result;
    }
    
    public ResponseEntity<List<LogStore>>  getAllLogs(String workflowInstanceId, String phaseId, String actionId, String serviceId){

        // Get userId
        UserInfo userInfo = IdentityUtils.getUserInfoFromAccessToken();
        String userId=userInfo.getUserId();

        ResponseEntity<List<LogStore>> result = null;
        List<LogStore> logsList=null;

        try {
            if(workflowInstanceId!=null && phaseId!=null && actionId!=null) {
                if (serviceId!=null) {
                    logsList = (List<LogStore>) this.logRepository.findByUserIdAndWorkflowInstanceIdAndPhaseIdAndActionIdAndServiceId(userId, 
                        workflowInstanceId, phaseId, actionId, serviceId);
                    
                } else {
                    logsList = (List<LogStore>) this.logRepository.findByUserIdAndWorkflowInstanceIdAndPhaseIdAndActionId(userId, 
                        workflowInstanceId, phaseId, actionId);
                }
            } else {
                logsList = (List<LogStore>) this.logRepository.findAllByUserIdAndWorkflowInstanceIdAndPhaseIdAndActionIdAndServiceId(userId, 
                    workflowInstanceId, phaseId, actionId, serviceId);
            }
            

            result=new ResponseEntity<List<LogStore>>(logsList,HttpStatus.OK);
        }
        catch(Exception e) {
            result=new ResponseEntity<>(logsList,HttpStatus.NOT_FOUND);
        }
        return result;
    }


    public void  saveLogInDB (String description, 
        String workflowInstanceId, String phaseId, String actionId, String serviceId, String userId, boolean providedByUser) {
        
        LogStore logStore; 

        logStore = new LogStore();
        logStore.setDescription(description);
        logStore.setWorkflowInstanceId(workflowInstanceId);
        logStore.setPhaseId(phaseId);
        logStore.setActionId(actionId);
        logStore.setServiceId(serviceId);
        logStore.setCreatedOn(Instant.now().toString());
        logStore.setUserId(userId);
        logStore.setUpdatedByUser(providedByUser);

        this.logRepository.saveAndFlush(logStore);
        return;
    }
    

    public ResponseEntity<StatisticsInfo>  getStatisticsInfo(){
        ResponseEntity<StatisticsInfo> result = null;
        StatisticsInfo statisticsInfo=new StatisticsInfo();
        try {
            statisticsInfo.setNumUsersStartedJourney(this.logRepository.getCountOfUsersStartedJourney());
            statisticsInfo.setNumStartedJourney(this.logRepository.getCountOfStartedJourneys());
            result=new ResponseEntity<StatisticsInfo>(statisticsInfo,HttpStatus.OK);
        }
        catch(Exception e) {
            result=new ResponseEntity<>(statisticsInfo,HttpStatus.NOT_FOUND);
        }
        return result;
    }
    
}
