package com.tecnalia.across.ujse.service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;

import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.tecnalia.across.ujse.exception.DataConsentRequiredException;
import com.tecnalia.across.ujse.exception.MissingInputsException;
import com.tecnalia.across.ujse.exception.PolicyNotApplyException;
import com.tecnalia.across.ujse.exception.RestContentNotSupportedException;
import com.tecnalia.across.ujse.exception.RestMethodNotSupportedException;
import com.tecnalia.across.ujse.external.ServiceCatalogueENG;
import com.tecnalia.across.ujse.external.TransparencyDashboard;
import com.tecnalia.across.ujse.model.ActionServiceStatusOutput;
import com.tecnalia.across.ujse.model.ActionStatusOutput;
import com.tecnalia.across.ujse.model.ExecuteStepOutput;
import com.tecnalia.across.ujse.model.OutputDataElem;
import com.tecnalia.across.ujse.model.ServiceInfo;
import com.tecnalia.across.ujse.model.ServiceStore;
import com.tecnalia.across.ujse.model.UserInfo;
import com.tecnalia.across.ujse.model.UserWorkflowModel;
import com.tecnalia.across.ujse.model.store.UserWorkflowStore;
import com.tecnalia.across.ujse.model.WorkflowStatusOutput;
import com.tecnalia.across.ujse.model.store.WorkflowPhaseActionServiceStore;
import com.tecnalia.across.ujse.model.store.WorkflowStore;
import com.tecnalia.across.ujse.model.store.WorkflowPhaseActionStore;
import com.tecnalia.across.ujse.repository.LogRepository;
import com.tecnalia.across.ujse.repository.UserWorkflowRepository;
import com.tecnalia.across.ujse.repository.WorkflowPhaseActionRepository;
import com.tecnalia.across.ujse.repository.WorkflowRepository;
import com.tecnalia.across.ujse.utils.WorkflowUtils;
import java.util.Map;
import com.tecnalia.across.ujse.repository.WorkflowPhaseActionServiceRepository;
import com.tecnalia.across.ujse.utils.IdentityUtils;
import java.nio.charset.StandardCharsets;
import java.util.UUID;


@Service
public class WorkflowExecutionManagementService {
	
    private static final Logger log = LoggerFactory.getLogger(WorkflowExecutionManagementService.class);

    @Autowired

    private  UserWorkflowRepository userWorkflowRepository;
    private  WorkflowRepository workflowRepository;
    private WorkflowPhaseActionServiceRepository workflowPhaseActionServiceRepository;
    private WorkflowPhaseActionRepository workflowPhaseActionRepository;
    private LogRepository logRepository;


    @Value("${service.catalog.uri.getServiceInfoENG}")
    String getServiceInfoUrlENG="";
    @Value("${service.catalog.uri.getServicesInfoENG}")
    String getServicesUrlENG="";
    
    @Value("${transparency.dashboard.main.uri}")
    String transparencyDashboardMainUri = "";
    
    @Value("${transparency.dashboard.consent.uri}")
    String consentPart = "";
    
    @Value("${transparency.dashboard.services.withoutconsent.uri}")
    String servicesWithoutConsentUrlPart = "";
    
    @Value("${transparency.dashboard.selectServicesByUser.uri}")
    String selectServicesByUserUrlPart = "";
    
    @Value("${transparency.dashboard.consent.activated.uri}")
    String consentActivatedUrlPart = "";
    
    @Value("${transparency.dashboard.consent.info.uri}")
    String consentInfoUrlPart = "";
    
    @Value("${transparency.dashboard.consentevents.uri}")
    String consentEventsPart = "";
    
    @Value("${transparency.dashboard.datausage.uri}")
    String dataUsageUrlPart = "";
   
    //TODO: Remove these variables. No need to ask for token to the TD. It is already provided.
    @Value("${spring.security.oauth2.resourceserver.jwt.issuer-uri}")
    String oauthUrl = "";

    @Value("${token.endpoint.url.path}")
    String oauthTokenUrlPart = "";        
  
    @Value("${datausage.enforcement.uri}")
    String datausagecontrolEnforcementUri="";
    
    @Value("${datausage.api.rest}")
    String datausagecontrolApiRest="";
    
    @Value("${datausage.policy.template}")
    String datausagePolicyTemplate="";
    
    @Value("${datausage.usagecontrol.user}")
    String datausagecontrolUser="";
    
    @Value("${datausage.usagecontrol.checkdatausagepolicies}")
    String datausagecontrolCheckdatausagepolicies="";
  
    @Autowired
    public WorkflowExecutionManagementService(UserWorkflowRepository userWorkflowRepository, WorkflowRepository workflowRepository,WorkflowPhaseActionServiceRepository workflowPhaseActionServiceRepository,WorkflowPhaseActionRepository workflowPhaseActionRepository, LogRepository logRepository) throws IllegalArgumentException {
 

        if (userWorkflowRepository == null)
            throw new IllegalArgumentException("userWorkflowRepository cannot be null.");       
        this.userWorkflowRepository = userWorkflowRepository;
        
        if (workflowRepository == null)
            throw new IllegalArgumentException("bpmnRepository cannot be null.");       
        this.workflowRepository = workflowRepository;
        
        
        if (workflowPhaseActionServiceRepository == null)
            throw new IllegalArgumentException("workflowPhaseActionServiceRepository cannot be null.");       
        this.workflowPhaseActionServiceRepository = workflowPhaseActionServiceRepository;
        
        
        if (workflowPhaseActionRepository == null)
            throw new IllegalArgumentException("workflowPhaseActionRepository cannot be null.");       
        this.workflowPhaseActionRepository = workflowPhaseActionRepository;
        
        
        if (logRepository == null)
            throw new IllegalArgumentException("logRepository cannot be null.");       
        this.logRepository = logRepository;
    }

	 
    public ResponseEntity<?> createUserWorkflowInstance(String countryOrigin, String countryDestination, String workflowType,String name) throws IOException {

        ResponseEntity<?> result = null;
        String message="";
        UserWorkflowStore userWorkflowStore;
        UserWorkflowModel userWorkflowModel = null;
        WorkflowUtils wUtils= new WorkflowUtils();
        boolean bKilledFinished=false;
        try {
         
            UserInfo userInfo = IdentityUtils.getUserInfoFromAccessToken();
            UUID userId=UUID.fromString(userInfo.getUserId());
            System.out.println(userId +":::"+countryOrigin+":::"+countryDestination+":::"+workflowType);
            Optional<WorkflowStore> bCheckExistsWorkflow=this.workflowRepository.findByCountryOriginAndCountryDestinationAndWorkflowType(countryOrigin.toUpperCase(), countryDestination.toUpperCase(), workflowType.toUpperCase());
        
            WorkflowStore workflowStore;

            if (bCheckExistsWorkflow.isPresent()) {
                //check if exist another instance of workflow by the userId    
                if (countryDestination!=null)
                    countryDestination=countryDestination.toUpperCase();
                if (countryOrigin!=null)
                    countryOrigin=countryOrigin.toUpperCase();
                if (workflowType!=null)
                    workflowType=workflowType.toUpperCase();
                
                
                List<UserWorkflowStore> listStatusWorkflow=(List<UserWorkflowStore>) this.userWorkflowRepository.findAByUserIdAndCountryOriginAndCountryDestinationAndWorkflowTypeAndWorkflowStatus(userId, countryOrigin, countryDestination, workflowType, "running");
                if ((listStatusWorkflow == null) || (listStatusWorkflow.isEmpty())){            
                        
                    workflowStore = (WorkflowStore)bCheckExistsWorkflow.get();
                    UUID workflowId=workflowStore.getWorkflowId();


                    byte[] jsonFile=workflowStore.getJSONfileInternal();
                    String content = new String (jsonFile);
                    //System.out.println("Internal Content:"+content);


                    HashMap<String,Object> workflowInfo=wUtils.readJSONFile(jsonFile);             
                    // Set the channel of the services in the workflow content
                    String engine_wf_content = wUtils.setWorkflowContentServicesInfo(getServiceInfoUrlENG, workflowInfo.get("workflowContent").toString());
                    
                    userWorkflowStore=new UserWorkflowStore();
                    userWorkflowStore.setName(name);
                    userWorkflowStore.setDescription(workflowInfo.get("description").toString());
                    userWorkflowStore.setUserId(userId);
                    userWorkflowStore.setWorkflowId(workflowId);
                    userWorkflowStore.setWorkflowType(workflowType.toUpperCase());
                    userWorkflowStore.setCountryOrigin(countryOrigin.toUpperCase());
                    userWorkflowStore.setCountryDestination(countryDestination.toUpperCase());                   
                    userWorkflowStore.setWorkflowStatus("running");
                    userWorkflowStore.setTrust(workflowStore.getTrust());
                    userWorkflowStore.setWorkflowContent(engine_wf_content);
                    userWorkflowStore.setCreatedAt(System.currentTimeMillis());
                    this.userWorkflowRepository.save(userWorkflowStore);

                    userWorkflowModel=new UserWorkflowModel();
                    userWorkflowModel.setName(name);
                    userWorkflowModel.setDescription(workflowInfo.get("description").toString());
                    userWorkflowModel.setUserId(userId);
                    userWorkflowModel.setWorkflowInstanceId(userWorkflowStore.getWorkflowInstanceId());
                    userWorkflowModel.setWorkflowId(workflowId);
                    userWorkflowModel.setWorkflowType(workflowType.toUpperCase());
                    userWorkflowModel.setCountryOrigin(countryOrigin.toUpperCase());
                    userWorkflowModel.setCountryDestination(countryDestination.toUpperCase());                 
                    userWorkflowModel.setWorkflowStatus("running");
                    userWorkflowModel.setTrust(workflowStore.getTrust());
                    userWorkflowModel.setWorkflowContent(engine_wf_content);
                    userWorkflowModel.setCreatedAt(System.currentTimeMillis());


                    HashMap<UUID,HashMap<UUID,ArrayList<String>>> phaseActionIdServicesEquivalences=new HashMap<>();
                    phaseActionIdServicesEquivalences=(HashMap<UUID,HashMap<UUID,ArrayList<String>>>) workflowInfo.get("actionServices");
                    this.setWorkflowPhaseActionServices(phaseActionIdServicesEquivalences,userWorkflowModel.getWorkflowInstanceId());

                    HashMap<UUID,ArrayList<UUID>> phaseActions=new HashMap<>();
                    phaseActions=(HashMap<UUID,ArrayList<UUID>>)workflowInfo.get("phaseActions");
                    this.setWorkflowPhaseAction(phaseActions, userWorkflowModel.getWorkflowInstanceId());


                    //get PersonalData related service from ServiceCatalog
                    ServiceCatalogueENG serviceCatalogueENG = new ServiceCatalogueENG();
                    List<ServiceInfo> returnlist=serviceCatalogueENG.getPersonalDataServices(getServicesUrlENG,(ArrayList<String>) workflowInfo.get("servicesId"),userWorkflowStore.getWorkflowInstanceId()); 


                    //return list of services without consent given
                    if (returnlist.size()>0){
                        String urlServicesWithoutConsent=transparencyDashboardMainUri+consentPart+userId+servicesWithoutConsentUrlPart;
                        TransparencyDashboard transparencyDashboard= new TransparencyDashboard();
                        List<ServiceInfo> listWithoutConsent;
                        listWithoutConsent = new ArrayList<>();
                        listWithoutConsent=transparencyDashboard.getNotConsentGivenServicesList(urlServicesWithoutConsent, returnlist, oauthUrl, oauthTokenUrlPart);

                        ObjectMapper objectMapper = new ObjectMapper();
                        objectMapper.enable(SerializationFeature.INDENT_OUTPUT);
                        String servicesWithoutConsentAsJson = objectMapper.writeValueAsString(listWithoutConsent);
                        userWorkflowModel.setServicesWithoutConsent(servicesWithoutConsentAsJson);


                        //insert automatically services in select services section in transparency dashboard
                        if (!listWithoutConsent.isEmpty()){
                            String urlSelectServices=transparencyDashboardMainUri+consentPart+userId+selectServicesByUserUrlPart;
                            boolean output=transparencyDashboard.selectServicesByUser(urlSelectServices,listWithoutConsent, oauthUrl, oauthTokenUrlPart);
                        }        
                    }
                    message=String.format("Workflow instance %s created from workflow %s", userWorkflowModel.getWorkflowInstanceId().toString(), userWorkflowModel.getWorkflowId());
                    LogManagementService logService = new LogManagementService(logRepository);
                    logService.saveLog(message, userWorkflowModel.getWorkflowInstanceId().toString(), null, null, null);
                    result=new ResponseEntity<UserWorkflowModel>(userWorkflowModel,HttpStatus.OK); 
                }
                else {
                    //One instance already running
                    message = "There is already one instance running for this Workflow";
                    JSONObject messageJson = new JSONObject();
                    messageJson.put("message", message);                       
                    result=new ResponseEntity<>(messageJson.toString(),HttpStatus.BAD_REQUEST);
                }
            } 
            else {
                //there is no workflowIdInstance for countryOrigin, countryDestination, workflowType
                message = "There is no workflow type for the selected parameters";
                JSONObject messageJson = new JSONObject();
                messageJson.put("message", message);                       
                result=new ResponseEntity<>(messageJson.toString(),HttpStatus.NOT_FOUND);
            }
        }
        catch(IOException | IllegalArgumentException e) {
            System.out.println(e.getMessage());
            userWorkflowModel = null;
            result=new ResponseEntity<>(userWorkflowModel,HttpStatus.BAD_REQUEST);
        }
        return result;
    }                          
                 


    public ResponseEntity<List<UserWorkflowModel>>  getUserWorkflowInstanceById(UUID workflowInstanceId){
		 

        ResponseEntity<List<UserWorkflowModel>> result = null;

        List<UserWorkflowStore> listUserWorkflow=null;
        List<UserWorkflowModel> listUserWorkflowModel=null;

        try {
            listUserWorkflow=(List<UserWorkflowStore>) this.userWorkflowRepository.findAllByWorkflowInstanceId(workflowInstanceId);

            if (listUserWorkflow!=null) {
                listUserWorkflowModel=new ArrayList();
                for (UserWorkflowStore userWf : listUserWorkflow) { 
                    UserWorkflowModel model=this.setWorkflowInstanceInfo(userWf);
                    listUserWorkflowModel.add(model);
                }
                result=new ResponseEntity<List<UserWorkflowModel>>(listUserWorkflowModel,HttpStatus.OK);
            }
            else {
                result=new ResponseEntity<List<UserWorkflowModel>>(listUserWorkflowModel,HttpStatus.NOT_FOUND);
            }
        }
        catch(Exception e) {
                result=new ResponseEntity<List<UserWorkflowModel>>(listUserWorkflowModel,HttpStatus.NOT_FOUND);
        }
        return result;
    }
	 
    public ResponseEntity<List<UserWorkflowModel>>  getUserWorkflowsInstances(String countryOrigin, String countryDestination, String workflowType){

        ResponseEntity<List<UserWorkflowModel>> result = null;
        List<UserWorkflowStore> listUserWorkflow=null;
        List<UserWorkflowModel> listUserWorkflowModel=null;

        try {
            UserInfo userInfo = IdentityUtils.getUserInfoFromAccessToken();
            UUID userId=UUID.fromString(userInfo.getUserId());
            
            if (countryDestination!=null)
                countryDestination=countryDestination.toUpperCase();
            if (countryOrigin!=null)
                countryOrigin=countryOrigin.toUpperCase();
            if (workflowType!=null)
                workflowType=workflowType.toUpperCase();
            
            listUserWorkflow=(List<UserWorkflowStore>) this.userWorkflowRepository.findAllByUserIdAndCountryOriginAndCountryDestinationAndWorkflowType(userId, countryOrigin,  countryDestination,  workflowType);

            if (listUserWorkflow!=null) {
                listUserWorkflowModel=new ArrayList();
               for (UserWorkflowStore userWf : listUserWorkflow) { 
                    UserWorkflowModel model=this.setWorkflowInstanceInfo(userWf);
                    listUserWorkflowModel.add(model);
                }

                result=new ResponseEntity<>(listUserWorkflowModel,HttpStatus.OK);

            }
            else {
                result=new ResponseEntity<>(listUserWorkflowModel,HttpStatus.NOT_FOUND);
            }
        }
        catch(Exception e) {
                result=new ResponseEntity<>(listUserWorkflowModel,HttpStatus.NOT_FOUND);
        }
        return result;
    }
	 
   
   
    public ResponseEntity<?> getWorkflowStatus(UUID workflowInstanceId){
    
        WorkflowStatusOutput wStatus=null;
        ResponseEntity<?> result = null;
        String status="";
   
        try {
            Optional<UserWorkflowStore> bCheckExistsUserWorkflow=this.userWorkflowRepository.findByWorkflowInstanceId(workflowInstanceId);
            UserWorkflowStore userWorkflowStore;
            if (bCheckExistsUserWorkflow.isPresent()) {
                userWorkflowStore = (UserWorkflowStore)bCheckExistsUserWorkflow.get();
                wStatus=new WorkflowStatusOutput();
                wStatus.setWorkflowInstanceId(workflowInstanceId);
                wStatus.setStatus(userWorkflowStore.getWorkflowStatus());
             result=new ResponseEntity<WorkflowStatusOutput> (wStatus,HttpStatus.OK);
            }
            else {
                String message = String.format("Workflow instance %s does not exist", workflowInstanceId.toString());
                JSONObject messageJson = new JSONObject();
                messageJson.put("message", message);                       
                result=new ResponseEntity<> (messageJson.toString(),HttpStatus.NOT_FOUND);
            }
        }
        catch (Exception e) {
           result=new ResponseEntity<WorkflowStatusOutput> ( wStatus,HttpStatus.NOT_FOUND);
        }
        return result;
    }

    public ResponseEntity<?> getServiceInfo(String serviceId) {
        ResponseEntity<?> result;
        ServiceStore serviceStore=null;
        try {
            ServiceCatalogueENG serviceCatalogueENG = new ServiceCatalogueENG();
            serviceStore = serviceCatalogueENG.getServiceInfo(serviceId, getServiceInfoUrlENG); 			    		 
            result=new ResponseEntity<ServiceStore> (serviceStore,HttpStatus.OK);
        } catch (JSONException | IOException e) {
            String message = String.format("Service %s does not exist", serviceId.toString());
            JSONObject messageJson = new JSONObject();
            messageJson.put("message", message);                       
            result=new ResponseEntity<> (messageJson.toString(),HttpStatus.NOT_FOUND);
        }
        return result;
    }

    public ResponseEntity<?> serviceFinishedOffline(UUID workflowInstanceId,UUID phaseId,UUID actionId, String serviceId) {

        ResponseEntity<?> result=null;
        ActionServiceStatusOutput actionServiceStatusOutput = null;
        String message="";
        System.out.println(workflowInstanceId +":::"+workflowInstanceId);
        WorkflowUtils wUtils= new WorkflowUtils();
        try {
            Optional<UserWorkflowStore> bCheckExistsUserWorkflow=this.userWorkflowRepository.findByWorkflowInstanceId(workflowInstanceId);
            UserWorkflowStore userWorkflowStore;
            if (bCheckExistsUserWorkflow.isPresent()) {
                userWorkflowStore = (UserWorkflowStore)bCheckExistsUserWorkflow.get();
                if ((!userWorkflowStore.getWorkflowStatus().equals("finished"))&&(!userWorkflowStore.getWorkflowStatus().equals("killed"))){
                    //find by workflow/stepId,serviceId
                    Optional<WorkflowPhaseActionServiceStore> bCheckWorkflowServiceStatus=this.workflowPhaseActionServiceRepository.findByWorkflowInstanceIdAndPhaseIdAndActionIdAndServiceId(workflowInstanceId, phaseId, actionId, serviceId);
                    WorkflowPhaseActionServiceStore workflowPhaseActionServiceStore;
                    if (bCheckWorkflowServiceStatus.isPresent()) {
                        workflowPhaseActionServiceStore = (WorkflowPhaseActionServiceStore)bCheckWorkflowServiceStatus.get();
                        if (!workflowPhaseActionServiceStore.getStatus().equals("finished")){

                            //change workflowContent 
                            String workflowContent=userWorkflowStore.getWorkflowContent();
                            String wf_json_str = wUtils.setWorkflowContentServiceStatus(phaseId.toString(), actionId.toString(), serviceId, "finished", null, workflowContent);
                            userWorkflowStore.setWorkflowContent(wf_json_str);
                            this.userWorkflowRepository.saveAndFlush(userWorkflowStore);
                            //change status
                            workflowPhaseActionServiceStore.setStatus("finished");
                            this.workflowPhaseActionServiceRepository.saveAndFlush(workflowPhaseActionServiceStore);
                            
                            
                            actionServiceStatusOutput=new ActionServiceStatusOutput();
                            actionServiceStatusOutput.setWorkflowInstanceId(workflowInstanceId);
                            actionServiceStatusOutput.setPhaseId(phaseId);
                            actionServiceStatusOutput.setActionId(actionId);
                            actionServiceStatusOutput.setServiceId(serviceId);
                            actionServiceStatusOutput.setStatus("finished");
                            result=new ResponseEntity<ActionServiceStatusOutput> (actionServiceStatusOutput,HttpStatus.OK);

                            message=String.format("Service status set as finished");
                            LogManagementService logService = new LogManagementService(logRepository);
                            logService.saveLog(message, workflowInstanceId.toString(), phaseId.toString(), actionId.toString(), serviceId); 
                        }
                        else{
                            message = String.format("Service %s in workflow instance %s, phase %s and action %s already finished", 
                                    serviceId, workflowInstanceId.toString(), phaseId.toString(), actionId.toString());
                            JSONObject messageJson = new JSONObject();
                            messageJson.put("message", message);                       
                            result=new ResponseEntity<> (messageJson.toString(),HttpStatus.BAD_REQUEST);
                        }
                    }
                    else{
                        message = String.format("Service %s in workflow instance %s, phase %s and action %s does not exist", 
                                serviceId, workflowInstanceId.toString(), phaseId.toString(), actionId.toString());
                        JSONObject messageJson = new JSONObject();
                        messageJson.put("message", message);                       
                        result=new ResponseEntity<> (messageJson.toString(),HttpStatus.NOT_FOUND);
                    }
                }    
                else{
                    message = String.format("Workflow %s is not running",  workflowInstanceId.toString());
                    JSONObject messageJson = new JSONObject();
                    messageJson.put("message", message);                       
                    result=new ResponseEntity<> (messageJson.toString(),HttpStatus.BAD_REQUEST);
                }
            }
            else {
                message = String.format("Workflow %s does not exist",  workflowInstanceId.toString());
                JSONObject messageJson = new JSONObject();
                messageJson.put("message", message);                       
                result=new ResponseEntity<> (messageJson.toString(),HttpStatus.NOT_FOUND);
            }
        }
        catch(Exception e) {
            message="An error happened during step finishing process"; 
            result=new ResponseEntity<ActionServiceStatusOutput> (actionServiceStatusOutput,HttpStatus.NOT_FOUND);
        }
        return result;
    }


    public ResponseEntity<?> setServiceStatus(UUID workflowInstanceId,UUID phaseId,UUID actionId, String serviceId, String status,
            String serviceOutput, String userId, boolean providedByUser) {

        ResponseEntity<?> result=null;
        ActionServiceStatusOutput actionServiceStatusOutput = null;
        String message="";
        System.out.println(workflowInstanceId +":::"+workflowInstanceId);
        WorkflowUtils wUtils= new WorkflowUtils();
        try {
            Optional<UserWorkflowStore> bCheckExistsUserWorkflow=this.userWorkflowRepository.findByWorkflowInstanceId(workflowInstanceId);
            UserWorkflowStore userWorkflowStore;
            if (bCheckExistsUserWorkflow.isPresent()) {
                userWorkflowStore = (UserWorkflowStore)bCheckExistsUserWorkflow.get();
                if ((!userWorkflowStore.getWorkflowStatus().equals("finished"))&&(!userWorkflowStore.getWorkflowStatus().equals("killed"))){
                    //find by workflow/stepId,serviceId
                    Optional<WorkflowPhaseActionServiceStore> bCheckWorkflowServiceStatus=this.workflowPhaseActionServiceRepository.findByWorkflowInstanceIdAndPhaseIdAndActionIdAndServiceId(workflowInstanceId, phaseId, actionId, serviceId);
                    WorkflowPhaseActionServiceStore workflowPhaseActionServiceStore;
                    if (bCheckWorkflowServiceStatus.isPresent()) {
                        workflowPhaseActionServiceStore = (WorkflowPhaseActionServiceStore)bCheckWorkflowServiceStatus.get();
                        if(serviceOutput==null)
                            // Do not update serviecOutput, if input is null
                            serviceOutput = workflowPhaseActionServiceStore.getServiceOutput();
                        if (wUtils.checkActionServiceStatusPossibleValues(status)){

                            //change workflowContent 
                            status = status.toLowerCase();
                            String workflowContent=userWorkflowStore.getWorkflowContent();
                            String wf_json_str = wUtils.setWorkflowContentServiceStatus(phaseId.toString(), actionId.toString(), serviceId, status, serviceOutput, workflowContent);
                            userWorkflowStore.setWorkflowContent(wf_json_str);
                            this.userWorkflowRepository.saveAndFlush(userWorkflowStore);
                            //change status
                            workflowPhaseActionServiceStore.setStatus(status);
                            workflowPhaseActionServiceStore.setServiceOutput(serviceOutput);
                            this.workflowPhaseActionServiceRepository.saveAndFlush(workflowPhaseActionServiceStore);
                            
                            
                            actionServiceStatusOutput=new ActionServiceStatusOutput();
                            actionServiceStatusOutput.setWorkflowInstanceId(workflowInstanceId);
                            actionServiceStatusOutput.setPhaseId(phaseId);
                            actionServiceStatusOutput.setActionId(actionId);
                            actionServiceStatusOutput.setServiceId(serviceId);
                            actionServiceStatusOutput.setStatus(status);
                            actionServiceStatusOutput.setServiceOutput(serviceOutput);
                            result=new ResponseEntity<ActionServiceStatusOutput> (actionServiceStatusOutput,HttpStatus.OK);

                            message=String.format("Service status set as %s", status);
                            LogManagementService logService = new LogManagementService(logRepository);
                            logService.saveLogInDB(message, workflowInstanceId.toString(), phaseId.toString(), actionId.toString(), serviceId, userId, providedByUser); 
                        }
                        else{
                            message = String.format("The status %s is not correct",  status);
                            JSONObject messageJson = new JSONObject();
                            messageJson.put("message", message);                       
                            result=new ResponseEntity<> (messageJson.toString(),HttpStatus.BAD_REQUEST);
                        }
                    }
                    else{
                        message = String.format("Service %s in workflow instance %s, phase %s and action %s does not exist", 
                                serviceId, workflowInstanceId.toString(), phaseId.toString(), actionId.toString());
                        JSONObject messageJson = new JSONObject();
                        messageJson.put("message", message);                       
                        result=new ResponseEntity<> (messageJson.toString(),HttpStatus.NOT_FOUND);
                    }
                }    
                else{
                    message = String.format("Workflow %s is not running",  workflowInstanceId.toString());
                    JSONObject messageJson = new JSONObject();
                    messageJson.put("message", message);                       
                    result=new ResponseEntity<> (messageJson.toString(),HttpStatus.BAD_REQUEST);
                }
            }
            else {
                message = String.format("Workflow %s does not exist",  workflowInstanceId.toString());
                JSONObject messageJson = new JSONObject();
                messageJson.put("message", message);                       
                result=new ResponseEntity<> (messageJson.toString(),HttpStatus.NOT_FOUND);
            }
        }
        catch(Exception e) {
            message="An error happened during step finishing process"; 
            result=new ResponseEntity<ActionServiceStatusOutput> (actionServiceStatusOutput,HttpStatus.NOT_FOUND);
        }
        return result;
    }


    public ResponseEntity<?> killWorkflow(UUID workflowInstanceId){

        ResponseEntity<?> result=null;
        WorkflowStatusOutput response=null;
        System.out.println(workflowInstanceId);
        try {
            Optional<UserWorkflowStore> bCheckExistsUserWorkflow=this.userWorkflowRepository.findByWorkflowInstanceId(workflowInstanceId);
            String message="";
            UserWorkflowStore userWorkflowStore;
            boolean bFound=false;
            if (bCheckExistsUserWorkflow.isPresent()) {

                userWorkflowStore = (UserWorkflowStore)bCheckExistsUserWorkflow.get();

                if (!userWorkflowStore.getWorkflowStatus().equals("finished")) {
                    userWorkflowStore.setWorkflowStatus("killed");
                    this.userWorkflowRepository.saveAndFlush(userWorkflowStore);
                    response=new WorkflowStatusOutput();
                    response.setWorkflowInstanceId(workflowInstanceId);
                    response.setStatus("killed");
                    result=new ResponseEntity<WorkflowStatusOutput> (response,HttpStatus.OK);
                    message=String.format("Workflow instance %s killed", workflowInstanceId.toString());
                    LogManagementService logService = new LogManagementService(logRepository);
                    logService.saveLog(message, workflowInstanceId.toString(), null, null, null);
               } 
               else {
                    response=new WorkflowStatusOutput();
                    response.setWorkflowInstanceId(workflowInstanceId);
                    response.setStatus("finished");
                    result=new ResponseEntity<WorkflowStatusOutput> (response,HttpStatus.BAD_REQUEST);
                }
            }
            else {
                message = String.format("Workflow instance %s does not exist", workflowInstanceId.toString());
                JSONObject messageJson = new JSONObject();
                messageJson.put("message", message);                       
                result=new ResponseEntity<> (messageJson.toString(),HttpStatus.NOT_FOUND);
            }
        }
        catch(IOException | IllegalArgumentException e) {
                 result=new ResponseEntity<WorkflowStatusOutput> (response,HttpStatus.NOT_FOUND);
        }
        return result;
    }

	 
    public ResponseEntity<?> executeService(UUID workflowInstanceId, UUID phaseId, UUID actionId,
            String serviceId, String token, List<MultipartFile> inputData ) throws IOException {
        //”, “stepStatus”:”pending/finished/info X missing/consent missing”}.
        ResponseEntity<?> result = null;
        //System.out.println(workflowInstanceId +"::::"+phaseId+":::"+inputData);
        WorkflowUtils wUtils= new WorkflowUtils();

        try {	 
            Optional<UserWorkflowStore> bCheckExistsUserWorkflow=this.userWorkflowRepository.findByWorkflowInstanceId(workflowInstanceId);
            UserWorkflowStore userWorkflowStore;
            boolean bFound=false;
            if (bCheckExistsUserWorkflow.isPresent()) {
                userWorkflowStore = (UserWorkflowStore)bCheckExistsUserWorkflow.get();
                //System.out.println(userWorkflowStore.getWorkflowStatus());
                boolean bFinished=userWorkflowStore.getWorkflowStatus().equals("finished");
                boolean bKilled=userWorkflowStore.getWorkflowStatus().equals("killed");
                
               
                //if workflowStatus!=finished or != killed
                if ((bFinished!=true) && (bKilled!=true)){
                    
                    //check step/service status into workflow
                    Optional<WorkflowPhaseActionServiceStore> bCheckWorkflowServiceStatus=this.workflowPhaseActionServiceRepository.findByWorkflowInstanceIdAndPhaseIdAndActionIdAndServiceId(workflowInstanceId, phaseId, actionId, serviceId);
                    WorkflowPhaseActionServiceStore workflowStepServiceStore;
                    if (bCheckWorkflowServiceStatus.isPresent()) {
                        workflowStepServiceStore = (WorkflowPhaseActionServiceStore)bCheckWorkflowServiceStatus.get();
                        if (!workflowStepServiceStore.getStatus().equals("finished")){
                            ServiceCatalogueENG serviceCatalogueENG = new ServiceCatalogueENG();
                            String status = WorkflowUtils.SERV_INITIAL_STATUS;
                            try {
                                //
                                //Look for service in Service Catalogue an if the result it is ok, status finished
                                //Invoke service
                                // - if Consent missing -> consent missing
                                // - if input data required -> info X required
                                // - if invocation OK -> status Finished


                                String checkConsentServiceUrl = transparencyDashboardMainUri+consentPart+userWorkflowStore.getUserId()+consentActivatedUrlPart+"?serviceId="+java.net.URLEncoder.encode(serviceId, StandardCharsets.UTF_8);
                                String consentInfoServiceUrl = transparencyDashboardMainUri+consentPart+userWorkflowStore.getUserId() + consentInfoUrlPart;
                                String logDataUsageServiceUrl = transparencyDashboardMainUri+consentEventsPart+userWorkflowStore.getUserId()+dataUsageUrlPart+"?serviceId="+java.net.URLEncoder.encode(serviceId, StandardCharsets.UTF_8);
                                String getPolicyPatternsUrl=datausagecontrolEnforcementUri+datausagecontrolApiRest+datausagePolicyTemplate+datausagecontrolUser+userWorkflowStore.getUserId()+datausagecontrolCheckdatausagepolicies+"?serviceId="+java.net.URLEncoder.encode(serviceId, StandardCharsets.UTF_8);

                                List<OutputDataElem> executionOutput = null;

                                ExecuteStepOutput executeOutput = serviceCatalogueENG.executeService(serviceId, token, inputData, 
                                                    getServiceInfoUrlENG, checkConsentServiceUrl, consentInfoServiceUrl, logDataUsageServiceUrl,getPolicyPatternsUrl, datausagecontrolEnforcementUri,
                                                    oauthUrl, oauthTokenUrlPart);
                                status = executeOutput.getStepStatus();
                                result=new ResponseEntity<ExecuteStepOutput> (executeOutput,HttpStatus.OK);
                            } catch(MissingInputsException | DataConsentRequiredException | PolicyNotApplyException 
                                    | RestContentNotSupportedException | RestMethodNotSupportedException e) {
                                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(e.getMessage());
                            }
                            
                            //change workflowContent 
                            String workflowContent=userWorkflowStore.getWorkflowContent();
                            String wf_json_str = wUtils.setWorkflowContentServiceStatus(phaseId.toString(), actionId.toString(), serviceId, status, null, workflowContent);
                            userWorkflowStore.setWorkflowContent(wf_json_str);
                            this.userWorkflowRepository.saveAndFlush(userWorkflowStore);
                            //change status
                            workflowStepServiceStore.setStatus(status);
                            this.workflowPhaseActionServiceRepository.saveAndFlush(workflowStepServiceStore);
                            String message=String.format("Service executed and %s", status);
                            LogManagementService logService = new LogManagementService(logRepository);
                            logService.saveLog(message, workflowInstanceId.toString(), phaseId.toString(), actionId.toString(), serviceId); 
                        }
                        else {
                            String message = String.format("Service %s in workflow instance %s, phase %s and action %s already finished", 
                                    serviceId, workflowInstanceId.toString(), phaseId.toString(), actionId.toString());
                            JSONObject messageJson = new JSONObject();
                            messageJson.put("message", message);                       
                            result=new ResponseEntity<> (messageJson.toString(),HttpStatus.BAD_REQUEST);
                            return result;
                        }
                    }
                    else {
                        //Service not in workflow
                        String message = String.format("Service %s in workflow instance %s, phase %s and action %s does not exist", 
                                serviceId, workflowInstanceId.toString(), phaseId.toString(), actionId.toString());
                        JSONObject messageJson = new JSONObject();
                        messageJson.put("message", message);                       
                        result=new ResponseEntity<> (messageJson.toString(),HttpStatus.NOT_FOUND);
                        return result;
                    }
                } else {
                    //Worflow not running
                    String message = String.format("Workflow %s is not running",  workflowInstanceId.toString());
                    JSONObject messageJson = new JSONObject();
                    messageJson.put("message", message);                       
                    result=new ResponseEntity<> (messageJson.toString(),HttpStatus.BAD_REQUEST);
                    return result;
                }
            } else {
                String message = String.format("Workflow %s does not exist",  workflowInstanceId.toString());
                JSONObject messageJson = new JSONObject();
                messageJson.put("message", message);                       
                result=new ResponseEntity<> (messageJson.toString(),HttpStatus.NOT_FOUND);
                return result;
            }

        } catch (JSONException e) {
            e.printStackTrace();
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("The workflow/userId doesn't exist");
        }
        return result;
    }
       

    public ResponseEntity<?>  setWorkflowStatus(UUID workflowInstanceId ,String status) {
        WorkflowStatusOutput wStatus=null;
        ResponseEntity<?> result = null;
        try{
            Optional<UserWorkflowStore> bCheckExistsUserWorkflow=this.userWorkflowRepository.findByWorkflowInstanceId(workflowInstanceId);
            UserWorkflowStore userWorkflowStore;
            if (bCheckExistsUserWorkflow.isPresent()) {
                userWorkflowStore = (UserWorkflowStore)bCheckExistsUserWorkflow.get();
                if (status.equals("killed") ||  status.equals("finished")){
                    userWorkflowStore.setWorkflowStatus(status);
                    this.userWorkflowRepository.saveAndFlush(userWorkflowStore);

                    wStatus=new WorkflowStatusOutput();
                    wStatus.setWorkflowInstanceId(workflowInstanceId);
                    wStatus.setStatus(userWorkflowStore.getWorkflowStatus());
                    result=new ResponseEntity<WorkflowStatusOutput> (wStatus,HttpStatus.OK);
                    String message=String.format("Status of workflow instance %s set as %s", workflowInstanceId.toString(), status);
                    LogManagementService logService = new LogManagementService(logRepository);
                    logService.saveLog(message, workflowInstanceId.toString(), null, null, null);    
                }
                else {
                    //Worflow not running
                    String message = String.format("The status %s is not correct",  status);
                    JSONObject messageJson = new JSONObject();
                    messageJson.put("message", message);                       
                    result=new ResponseEntity<> (messageJson.toString(),HttpStatus.BAD_REQUEST);
               }
            }
            else {
                String message = String.format("Workflow %s does not exist",  workflowInstanceId.toString());
                JSONObject messageJson = new JSONObject();
                messageJson.put("message", message);                       
                result=new ResponseEntity<> (messageJson.toString(),HttpStatus.NOT_FOUND);
            }
        }            
        catch (Exception e) {
           result=new ResponseEntity<WorkflowStatusOutput> (wStatus,HttpStatus.NOT_FOUND);
        }
        return result;
    }
	 

    public ResponseEntity<String> deleteWorkflowInstanceId(UUID workflowInstanceId) {
        ResponseEntity<String> result = null;
        WorkflowStatusOutput workflowStatus=null;
        List<WorkflowPhaseActionServiceStore> listWorkflowServiceStatus=null;
        String message="";
        try {                
            Optional<UserWorkflowStore> bCheckExistsUserWorkflow=this.userWorkflowRepository.findByWorkflowInstanceId(workflowInstanceId);
            UserWorkflowStore userWorkflowStore;
            boolean bFound=false;
            boolean toUpdateCurrentStep = false;
            if (bCheckExistsUserWorkflow.isPresent()) {
                userWorkflowStore = (UserWorkflowStore)bCheckExistsUserWorkflow.get();
                this.userWorkflowRepository.deleteByWorkflowInstanceId(workflowInstanceId);
                this.workflowPhaseActionServiceRepository.deleteByWorkflowInstanceId(workflowInstanceId);
                this.workflowPhaseActionRepository.deleteByWorkflowInstanceId(workflowInstanceId);
               
                message=String.format("Workflow instance %s deleted", workflowInstanceId.toString());
                LogManagementService logService = new LogManagementService(logRepository);
                logService.saveLog(message, workflowInstanceId.toString(), null, null, null);	                    
                result=new ResponseEntity<String>(message, HttpStatus.OK);              
	    } else {
                message=String.format("Workflow instance %s does not exist", workflowInstanceId.toString());
                result=new ResponseEntity<String>(message, HttpStatus.NOT_FOUND);
            }            
        }
        catch (Exception e) {     
            message=String.format("Workflow instance %s does not exist", workflowInstanceId.toString());
            result=new ResponseEntity<String>(message, HttpStatus.NOT_FOUND);
        }
        return result;
    }
    
    
    public ResponseEntity<?> workflowName(UUID workflowInstanceId, String name) {
        ResponseEntity<?> result = null;
        List<WorkflowPhaseActionServiceStore> listWorkflowServiceStatus=null;
        UserWorkflowStore userWorkflowStore = null;
        String message="";
        try {
                
            Optional<UserWorkflowStore> bCheckExistsUserWorkflow=this.userWorkflowRepository.findByWorkflowInstanceId(workflowInstanceId);
           
          
            if (bCheckExistsUserWorkflow.isPresent()) {
                userWorkflowStore = (UserWorkflowStore)bCheckExistsUserWorkflow.get();
                userWorkflowStore.setName(name);
                this.userWorkflowRepository.saveAndFlush(userWorkflowStore);
                message=String.format("Name of workflow instance %s set as %s", workflowInstanceId.toString(), name);
                LogManagementService logService = new LogManagementService(logRepository);
                logService.saveLog(message, workflowInstanceId.toString(), null, null, null);  
                result=new ResponseEntity<UserWorkflowStore>(userWorkflowStore, HttpStatus.OK);              
	    } else {
                message=String.format("Workflow instance %s does not exist", workflowInstanceId.toString());
                JSONObject messageJson = new JSONObject();
                messageJson.put("message", message);                       
                result=new ResponseEntity<String>(messageJson.toString(), HttpStatus.NOT_FOUND);
            }            
        }
        catch (IOException | IllegalArgumentException e) {     
           result=new ResponseEntity<> ( userWorkflowStore,HttpStatus.NOT_FOUND);           
        }
        return result;
    }

       
    public ResponseEntity<?> setActionStatus(UUID workflowInstanceId,UUID phaseId,UUID actionId, String status) {
        ResponseEntity<?> result = null;
        ActionStatusOutput actionStatusOutput = null;
        WorkflowUtils wUtils= new WorkflowUtils();
 
        String message="";
        try {
            
            Optional<UserWorkflowStore> bCheckExistsUserWorkflow=this.userWorkflowRepository.findByWorkflowInstanceId(workflowInstanceId);
            UserWorkflowStore userWorkflowStore;
 
            if (bCheckExistsUserWorkflow.isPresent()) {
                userWorkflowStore = (UserWorkflowStore)bCheckExistsUserWorkflow.get();
                if ((!userWorkflowStore.getWorkflowStatus().equals("finished"))&&(!userWorkflowStore.getWorkflowStatus().equals("killed"))){
          
                    Optional<WorkflowPhaseActionStore> bCheckExistsPhaseAction=this.workflowPhaseActionRepository.findByWorkflowInstanceIdAndPhaseIdAndActionId(workflowInstanceId, phaseId, actionId);
                    WorkflowPhaseActionStore workflowPhaseActionStore;
                    if (bCheckExistsPhaseAction.isPresent()) {
                        workflowPhaseActionStore = (WorkflowPhaseActionStore)bCheckExistsPhaseAction.get();
                        if (wUtils.checkActionServiceStatusPossibleValues(status)){
                            //change workflowContent
                            status = status.toLowerCase();
                            String workflowContent=userWorkflowStore.getWorkflowContent();
                            String wf_json_str = wUtils.setWorkflowContentActionStatus(actionId.toString(), status, workflowContent);
                            userWorkflowStore.setWorkflowContent(wf_json_str);
                            this.userWorkflowRepository.saveAndFlush(userWorkflowStore);
                            //change status
                            workflowPhaseActionStore.setStatus(status);
                            this.workflowPhaseActionRepository.saveAndFlush(workflowPhaseActionStore);

                            actionStatusOutput=new ActionStatusOutput();
                            actionStatusOutput.setWorkflowInstanceId(workflowInstanceId);
                            actionStatusOutput.setPhaseId(phaseId);
                            actionStatusOutput.setActionId(actionId);
                            actionStatusOutput.setStatus(status);
                            
                            result=new ResponseEntity<ActionStatusOutput> (actionStatusOutput,HttpStatus.OK);

                            message=String.format("Action status set as %s", status);
                            LogManagementService logService = new LogManagementService(logRepository);
                            logService.saveLog(message, workflowInstanceId.toString(), phaseId.toString(), actionId.toString(), null); 
                        }
                        else{
                            message = String.format("The status %s is not correct",  status);
                            JSONObject messageJson = new JSONObject();
                            messageJson.put("message", message);                       
                            result=new ResponseEntity<> (messageJson.toString(),HttpStatus.BAD_REQUEST);
                       }
                    }
                    else {
                        message = String.format("Action %s in workflow instance %s, phase %s does not exist", 
                                actionId.toString(), workflowInstanceId.toString(), phaseId.toString());
                        JSONObject messageJson = new JSONObject();
                        messageJson.put("message", message);                       
                        result=new ResponseEntity<> (messageJson.toString(),HttpStatus.NOT_FOUND);                       
                    }
                }
                else {
                    message = String.format("Workflow %s is not running",  workflowInstanceId.toString());
                    JSONObject messageJson = new JSONObject();
                    messageJson.put("message", message);                       
                    result=new ResponseEntity<> (messageJson.toString(),HttpStatus.BAD_REQUEST);                   
                }
            }
            else {
                message = String.format("Workflow %s does not exist",  workflowInstanceId.toString());
                JSONObject messageJson = new JSONObject();
                messageJson.put("message", message);                       
                result=new ResponseEntity<> (messageJson.toString(),HttpStatus.NOT_FOUND);                
            }
        }
        catch (IOException | IllegalArgumentException e) {
           result=new ResponseEntity<ActionStatusOutput> (actionStatusOutput,HttpStatus.NOT_FOUND);
        }
        return result;
    }
    


    /////////////////////
    // Helping methods
    /////////////////////
    private UserWorkflowModel setWorkflowInstanceInfo(UserWorkflowStore userWf) throws Exception{
        UserWorkflowModel model=new UserWorkflowModel();
        model.setUserId(userWf.getUserId());
        model.setWorkflowInstanceId(userWf.getWorkflowInstanceId());
        model.setWorkflowId(userWf.getWorkflowId());
        model.setWorkflowType(userWf.getWorkflowType());
        model.setCountryOrigin(userWf.getCountryOrigin());
        model.setCountryDestination(userWf.getCountryDestination());	         
        model.setWorkflowStatus(userWf.getWorkflowStatus());
        model.setTrust(userWf.getTrust());
        model.setWorkflowContent(userWf.getWorkflowContent());
        model.setCreatedAt(userWf.getCreatedAt());
        model.setName(userWf.getName());
        model.setDescription(userWf.getDescription());
        //associatedServices for Workflow
        ArrayList<String> servicesIdList = (ArrayList<String>)this.workflowPhaseActionServiceRepository.findDistinctByWorkflowInstanceIdAndPhaseIdAndActionId(userWf.getWorkflowInstanceId());
        

        //return list of services with Personal Data
        ServiceCatalogueENG serviceCatalogueENG = new ServiceCatalogueENG();
        List<ServiceInfo> personalDataServicesList=serviceCatalogueENG.getPersonalDataServices(getServicesUrlENG,servicesIdList,userWf.getWorkflowInstanceId());

        //return from the previous list the services without consent
        String urlServicesWithoutConsent=transparencyDashboardMainUri+consentPart+userWf.getUserId()+servicesWithoutConsentUrlPart;
        TransparencyDashboard transparencyDashboard= new TransparencyDashboard();
        List<ServiceInfo> listWithoutConsent=transparencyDashboard.getNotConsentGivenServicesList(urlServicesWithoutConsent, personalDataServicesList, oauthUrl, oauthTokenUrlPart);

        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper.enable(SerializationFeature.INDENT_OUTPUT);
        String servicesWithoutConsentAsJson = objectMapper.writeValueAsString(listWithoutConsent);
        model.setServicesWithoutConsent(servicesWithoutConsentAsJson);

        return model;
    }
    
    private void setWorkflowPhaseActionServices(HashMap<UUID,HashMap<UUID,ArrayList<String>>> phaseActionIdServicesEquivalences, UUID workflowInstanceId) {
        
        for (Map.Entry<UUID, HashMap<UUID,ArrayList<String>>> setPhaseAction : phaseActionIdServicesEquivalences.entrySet()) {

            // Printing all elements of a Map
            //System.out.println(setPhaseAction.getKey() + " = "+ setPhaseAction.getValue());

            HashMap<UUID,ArrayList<String>> actionServicesIds=setPhaseAction.getValue();
            for (Map.Entry<UUID, ArrayList<String>>  setActionService : actionServicesIds.entrySet()) {

            // Printing all elements of a Map
                //System.out.println(setActionService.getKey() + " = "+ setActionService.getValue());
                ArrayList<String> aServices=setActionService.getValue();
                for (int i=0;i<aServices.size();i++){
                
                    WorkflowPhaseActionServiceStore workflowfPhaseActionServiceStore=new WorkflowPhaseActionServiceStore();
                    workflowfPhaseActionServiceStore.setWorkflowInstanceId(workflowInstanceId);
                    workflowfPhaseActionServiceStore.setPhaseId(setPhaseAction.getKey());
                    workflowfPhaseActionServiceStore.setActionId(setActionService.getKey());
                    workflowfPhaseActionServiceStore.setServiceId(aServices.get(i));
                    workflowfPhaseActionServiceStore.setStatus(WorkflowUtils.SERV_INITIAL_STATUS);
                    this.workflowPhaseActionServiceRepository.save(workflowfPhaseActionServiceStore);
                }    
            }    
        }
    }
    
    private void setWorkflowPhaseAction(HashMap<UUID,ArrayList<UUID>> phaseActions, UUID workflowInstanceId) {
        
        for (Map.Entry<UUID, ArrayList<UUID>> setPhaseAction : phaseActions.entrySet()) {

            // Printing all elements of a Map
            //System.out.println(setPhaseAction.getKey() + " = "+ setPhaseAction.getValue());

            ArrayList<UUID> aActions=setPhaseAction.getValue();
            for (int i=0;i<aActions.size();i++) {

                UUID actionId=aActions.get(i);
                WorkflowPhaseActionStore workflowfPhaseActionStore=new WorkflowPhaseActionStore();
                workflowfPhaseActionStore.setWorkflowInstanceId(workflowInstanceId);
                workflowfPhaseActionStore.setPhaseId(setPhaseAction.getKey());
                workflowfPhaseActionStore.setActionId(aActions.get(i));
                workflowfPhaseActionStore.setStatus(WorkflowUtils.ACT_INITIAL_STATUS);
                this.workflowPhaseActionRepository.save(workflowfPhaseActionStore);
                    
            }    
        }
    }
       
}
