package com.tecnalia.across.ujse.service;


import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.jayway.jsonpath.DocumentContext;
import com.tecnalia.across.ujse.model.WorkflowModel;
import java.io.IOException;
import java.util.List;
import java.util.Optional;

import javax.transaction.Transactional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
//import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import com.tecnalia.across.ujse.model.store.WorkflowStore;
import com.tecnalia.across.ujse.model.workflow.engine.Action;
import com.tecnalia.across.ujse.model.workflow.modellingtool.MTJson;
import com.tecnalia.across.ujse.model.workflow.engine.InternalJson;
import com.tecnalia.across.ujse.model.workflow.engine.Phase;
import com.tecnalia.across.ujse.model.workflow.engine.Touchpoint;
import com.tecnalia.across.ujse.model.workflow.modellingtool.MTJson;
import com.tecnalia.across.ujse.repository.WorkflowRepository;
import com.tecnalia.across.ujse.utils.WorkflowUtils;
import java.io.ByteArrayInputStream;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.UUID;
import org.apache.commons.io.IOUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.MockMultipartFile;



@Service
public class WorkflowManagementService {
	
    private static final Logger log = LoggerFactory.getLogger(WorkflowManagementService.class);

    @Value("${service.catalog.uri.getServiceInfoENG}")
    String getServiceInfoUrlENG="";

    @Autowired
    private  WorkflowRepository workflowRepository;

    @Autowired
    public WorkflowManagementService(WorkflowRepository workflowRepository) throws IllegalArgumentException {

        if (workflowRepository == null)
            throw new IllegalArgumentException("bpmnRepository cannot be null.");       

        this.workflowRepository = workflowRepository;

    }
 
	 	
    @Transactional
    public ResponseEntity<String>  addOrUpdateWorkflow (MultipartFile JSONfileName) throws IOException {

        ResponseEntity<String> result=null;
        WorkflowUtils wUtils= new WorkflowUtils();


        try {

            //convert json FOKUS to jsonInternal  
            ByteArrayInputStream stream = new   ByteArrayInputStream(JSONfileName.getBytes());
            String content = IOUtils.toString(stream, "UTF-8");


            content=wUtils.removeDrawIdKeyFromJSON(content, "drawio_id");


            ObjectMapper mapper = new ObjectMapper();
            mapper.configure(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES, true);
            MTJson mtJson = mapper.readValue(content, MTJson.class);



            InternalJson intJSON=wUtils.convertInitialToInternalJson(content);
            System.out.println("Uploading workflow " + intJSON.getDestination_country()+intJSON.getOrigin_country());

            String internalContent = mapper.writerWithDefaultPrettyPrinter().writeValueAsString(intJSON);
            byte[] internalContentArrray = internalContent.getBytes();

            if (intJSON!=null){

                //System.out.println(mtJson.getDescription());

                String countryOrigin=mtJson.getOrigin_country();
                String countryDestination=mtJson.getDestination_country();
                String workflowType=mtJson.getWorkflow_type();

                Optional<WorkflowStore> bCheckExistsWorkflow = this.workflowRepository.findByCountryOriginAndCountryDestinationAndWorkflowType(countryOrigin.toUpperCase(), countryDestination.toUpperCase(),workflowType.toUpperCase());
                WorkflowStore workflowStore;

                //if exists, update
                String message="";
                if (bCheckExistsWorkflow.isPresent()) {

                    //message="JSON File has been updated";
                    workflowStore = (WorkflowStore)bCheckExistsWorkflow.get();
                    workflowStore.setCountryOrigin(countryOrigin.toUpperCase());
                    workflowStore.setCountryDestination(countryDestination.toUpperCase());          
                    workflowStore.setWorkflowType(workflowType.toUpperCase());
                    /*workflowStore.setWorkflowName(BPMNfileName.getOriginalFilename());
                    workflowStore.setBpMNfile(BPMNfileName.getBytes());   */
                    workflowStore.setJSONFileName(JSONfileName.getOriginalFilename());
                    workflowStore.setJSONfile(JSONfileName.getBytes());  
                    workflowStore.setJSONfileInternal(internalContentArrray);
                    this.workflowRepository.saveAndFlush(workflowStore);
                    message=bCheckExistsWorkflow.get().getWorkflowId().toString();

               } 
               //insert new workflow
               else {
                    //message="JSON File has been added";
                    workflowStore = new WorkflowStore();
                    workflowStore.setCountryOrigin(countryOrigin.toUpperCase());
                    workflowStore.setCountryDestination(countryDestination.toUpperCase());          
                    workflowStore.setWorkflowType(workflowType.toUpperCase());
                    /*workflowStore.setWorkflowName(BPMNfileName.getOriginalFilename());
                    workflowStore.setBpMNfile(BPMNfileName.getBytes());   */
                    workflowStore.setJSONFileName(JSONfileName.getOriginalFilename());
                    workflowStore.setJSONfile(JSONfileName.getBytes());  
                    workflowStore.setJSONfileInternal(internalContentArrray);

                    this.workflowRepository.saveAndFlush(workflowStore);
                    message=workflowStore.getWorkflowId().toString();

               }   
               result=new ResponseEntity<String>(message,HttpStatus.OK);
            }

        }
        catch (Exception e) {
            String message="An error happened during workflow upload process";
            result=new ResponseEntity<String>(message,HttpStatus.BAD_REQUEST);
        }

        return result;

    }


    @Transactional
    public ResponseEntity<String> deleteWorkflow(UUID workflowId) {

        ResponseEntity<String> result; 		  
        try{  

          Optional<WorkflowStore> bCheckExistsWorkflow = this.workflowRepository.findAllByWorkflowId(workflowId);//findById(workflowId);

          //if exists, delete
          if (bCheckExistsWorkflow.isPresent()) {

              this.workflowRepository.deleteByWorkflowId(workflowId);
              result=new ResponseEntity<String>("Workflow has been deleted", HttpStatus.OK);
           }
          else
              result= new ResponseEntity<String>( "Workflow doesn't exists",HttpStatus.NOT_FOUND);
        }
        catch(Exception e){

           result= new ResponseEntity<String>("An error happened during workflow deletion process",HttpStatus.BAD_REQUEST);
        }
        return result;
    }
	    
	    
    public ResponseEntity<List<WorkflowStore>> getAllWorkflows() {

        try{

            List<WorkflowStore> list=this.workflowRepository.findAll();
          
            return new ResponseEntity<List<WorkflowStore>>(this.workflowRepository.findAll(), HttpStatus.OK);  

        }catch (Exception e){
            return new ResponseEntity<List<WorkflowStore>>(this.workflowRepository.findAll(), HttpStatus.BAD_REQUEST);

        }
    }
		
    
    public ResponseEntity<byte[]> getWorkflowById(UUID workflowId) {

        try{  
            Optional<WorkflowStore> bCheckExistsWorkflow = this.workflowRepository.findById(workflowId);
            WorkflowStore workflowStore;

            if (bCheckExistsWorkflow.isPresent()) {

                workflowStore = (WorkflowStore)bCheckExistsWorkflow.get();

                byte[] file=workflowStore.getJSONfile();
                return ResponseEntity.ok()
                                .header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + workflowStore.getJSONFileName() + "\"")
                                .body(workflowStore.getJSONfile());

            }
            else {
               return ResponseEntity.notFound().build();
            }
        }
        catch(Exception e){

            return ResponseEntity.badRequest()
                        .body(null);
        }

    }

	
    public ResponseEntity<List<WorkflowModel>> getWorkflowsDescription() {
        ResponseEntity<List<WorkflowModel>> result = null;
        List<WorkflowModel> listWorkflowModel=null;

        try{

            List<WorkflowStore> workflowsList=this.workflowRepository.findAll();
          

            if (workflowsList!=null) {
                listWorkflowModel=new ArrayList();
               for (WorkflowStore workflowStore : workflowsList) { 
                    WorkflowModel model=this.setWorkflowInfo(workflowStore);
                    listWorkflowModel.add(model);
                }

                result=new ResponseEntity<>(listWorkflowModel,HttpStatus.OK);

            }
            else {
                result=new ResponseEntity<>(listWorkflowModel,HttpStatus.NOT_FOUND);
            }

            return result;  


        }
        catch(Exception e) {
            result = new ResponseEntity<>(listWorkflowModel,HttpStatus.BAD_REQUEST);
        }
        return result;
    }
		
    
    public ResponseEntity<WorkflowModel> getWorkflowDescriptionById(UUID workflowId) {
        try{  
            Optional<WorkflowStore> bCheckExistsWorkflow = this.workflowRepository.findById(workflowId);
            WorkflowStore workflowStore;

            if (bCheckExistsWorkflow.isPresent()) {
                workflowStore = (WorkflowStore)bCheckExistsWorkflow.get();

                WorkflowModel workflowModel=this.setWorkflowInfo(workflowStore);
                ResponseEntity<WorkflowModel> result=new ResponseEntity<WorkflowModel> (workflowModel,HttpStatus.OK);
                return result;
            }
            else {
               return ResponseEntity.notFound().build();
            }
        }
        catch(Exception e){
            return ResponseEntity.badRequest()
                        .body(null);
        }
    }

	
  /*  public ResponseEntity<byte[]> dowloadWorkflowFile (UUID workflowId){
        WorkflowStore workflowStore = null;
        try{  

            Optional<WorkflowStore> bCheckExistsWorkflow = this.workflowRepository.findById(workflowId);//findById(workflowId);

            //if exists delete
            if (bCheckExistsWorkflow.isPresent()) {

                workflowStore = (WorkflowStore)bCheckExistsWorkflow.get();

                byte[] file=workflowStore.getJSONfile();
                return ResponseEntity.ok()
                                .header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + workflowStore.getJSONFileName()+ "\"")
                                .body(workflowStore.getJSONfile());

             }
            else
                return ResponseEntity.notFound().build();
        }
        catch(Exception e){

        return ResponseEntity.badRequest()
                        .body(null);
        }

    }*/
	
    public ResponseEntity<byte[]> dowloadWorkflowFile (UUID workflowId){
        WorkflowStore workflowStore = null;
        try{  

            Optional<WorkflowStore> bCheckExistsWorkflow = this.workflowRepository.findById(workflowId);//findById(workflowId);

            //if exists delete
            if (bCheckExistsWorkflow.isPresent()) {

                workflowStore = (WorkflowStore)bCheckExistsWorkflow.get();

                byte[] file=workflowStore.getJSONfile();
                return ResponseEntity.ok()
                                .header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + workflowStore.getJSONFileName()+ "\"")
                                .body(workflowStore.getJSONfileInternal());

            }
            else
                return ResponseEntity.notFound().build();
        }
        catch(Exception e){
            return ResponseEntity.badRequest().body(null);
        }

    }
    
    
    /////////////////////
    // Helping methods
    /////////////////////
    private WorkflowModel setWorkflowInfo(WorkflowStore workflowStore) throws Exception{
        WorkflowModel workflowModel = new WorkflowModel();
        WorkflowUtils wUtils= new WorkflowUtils();

        byte[] jsonFile=workflowStore.getJSONfileInternal();
        HashMap<String,Object> workflowInfo=wUtils.readJSONFile(jsonFile);             
        // Set the channel of the services in the workflow content
        String engine_wf_content = wUtils.setWorkflowContentServicesInfo(getServiceInfoUrlENG, workflowInfo.get("workflowContent").toString());

        workflowModel=new WorkflowModel();
        workflowModel.setDescription(workflowInfo.get("description").toString());
        workflowModel.setWorkflowId(workflowStore.getWorkflowId());
        workflowModel.setWorkflowType(workflowStore.getWorkflowType().toUpperCase());
        workflowModel.setCountryOrigin(workflowStore.getCountryOrigin().toUpperCase());
        workflowModel.setCountryDestination(workflowStore.getCountryDestination().toUpperCase());                 
        workflowModel.setWorkflowContent(engine_wf_content);

        return workflowModel;
    }
    
    

    @Transactional
    public ResponseEntity<String>  updateWorkflow (UUID workflowId, boolean trust) {

        ResponseEntity<String> result; 		  
        try{  
            Optional<WorkflowStore> bCheckExistsWorkflow = this.workflowRepository.findAllByWorkflowId(workflowId);
            
            //if exists, modify it
            if (bCheckExistsWorkflow.isPresent()) {
                WorkflowStore workflowStore = (WorkflowStore)bCheckExistsWorkflow.get();
                workflowStore.setTrust(trust);
                this.workflowRepository.saveAndFlush(workflowStore);
                result=new ResponseEntity<String>(workflowId.toString(), HttpStatus.OK);
            }
            else
                result= new ResponseEntity<String>( "Workflow doesn't exists",HttpStatus.NOT_FOUND);
        }
        catch(Exception e){
            result= new ResponseEntity<String>("An error happened during workflow update process",HttpStatus.BAD_REQUEST);
        }
        return result;        
    }


    @Transactional
    public ResponseEntity<String>  getWorkflowTrust (UUID workflowId) {

        ResponseEntity<String> result; 		  
        try{  
            Optional<WorkflowStore> bCheckExistsWorkflow = this.workflowRepository.findAllByWorkflowId(workflowId);
            
            //if exists, modify it
            if (bCheckExistsWorkflow.isPresent()) {
                WorkflowStore workflowStore = (WorkflowStore)bCheckExistsWorkflow.get();
                boolean trust = workflowStore.getTrust();
                result=new ResponseEntity<String>(String.valueOf(trust), HttpStatus.OK);
            }
            else
                result= new ResponseEntity<String>( "Workflow doesn't exists",HttpStatus.NOT_FOUND);
        }
        catch(Exception e){
            result= new ResponseEntity<String>("An error happened when getting the workflow info",HttpStatus.BAD_REQUEST);
        }
        return result;        
    }
}
